# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, List, Union, Optional
from datetime import datetime
from typing_extensions import Literal, TypeAlias

from pydantic import Field as FieldInfo

from .._models import BaseModel
from .eval_definition import EvalDefinition

__all__ = [
    "Trace",
    "Eval",
    "EvalResults",
    "EvalResultsUnionMember0",
    "EvalResultsUnionMember1",
    "EvalResultsUnionMember2",
    "EvalResultsUnionMember2AnswerRelevancy",
    "EvalResultsUnionMember2AnswerRelevancyMetadata",
    "EvalResultsUnionMember2AnswerRelevancyMetadataQuestion",
    "EvalResultsUnionMember2AnswerRelevancyMetadataSimilarity",
    "EvalResultsUnionMember2ContextPrecision",
    "EvalResultsUnionMember2ContextPrecisionMetadata",
    "EvalResultsUnionMember2ContextRelevancy",
    "EvalResultsUnionMember2ContextRelevancyMetadata",
    "EvalResultsUnionMember2ContextRelevancyMetadataRelevantSentence",
    "EvalResultsUnionMember2Faithfulness",
    "EvalResultsUnionMember2FaithfulnessMetadata",
    "EvalResultsUnionMember2FaithfulnessMetadataFaithfulness",
    "EvalResultsUnionMember3",
    "EvalResultsUnionMember3Metadata",
    "EvalResultsUnionMember4",
    "EvalResultsUnionMember4Classification",
    "EvalResultsUnionMember5",
    "EvalResultsUnionMember5Missing",
    "Step",
    "StepLlmStartStepOutput",
    "StepLlmEndStepOutput",
    "StepLlmEndStepOutputUsage",
    "StepToolStepOutput",
    "StepRetrieverStepOutput",
    "StepLogStepOutput",
    "StepGroupStepOutput",
    "StepResponseStepOutput",
    "StepRequestStepOutput",
]


class EvalResultsUnionMember0(BaseModel):
    analysis: str

    clarity: float

    coherence: float

    engagingness: float

    naturalness: float

    relevance: float


class EvalResultsUnionMember1(BaseModel):
    analysis: str
    """
    A brief explanation for your rating, referring to specific aspects of the
    response and the query. Make sure that the explanation is formatted as markdown,
    and that it is easy to read and understand.
    """

    score: float
    """The score of the response based on the style guide from 1-5"""


class EvalResultsUnionMember2AnswerRelevancyMetadataQuestion(BaseModel):
    question: str


class EvalResultsUnionMember2AnswerRelevancyMetadataSimilarity(BaseModel):
    question: str

    score: float


class EvalResultsUnionMember2AnswerRelevancyMetadata(BaseModel):
    questions: List[EvalResultsUnionMember2AnswerRelevancyMetadataQuestion]

    similarity: List[EvalResultsUnionMember2AnswerRelevancyMetadataSimilarity]


class EvalResultsUnionMember2AnswerRelevancy(BaseModel):
    name: str

    score: float

    error: Optional[str] = None

    metadata: Optional[EvalResultsUnionMember2AnswerRelevancyMetadata] = None


class EvalResultsUnionMember2ContextPrecisionMetadata(BaseModel):
    reason: str

    verdict: int


class EvalResultsUnionMember2ContextPrecision(BaseModel):
    name: str

    score: float

    error: Optional[str] = None

    metadata: Optional[EvalResultsUnionMember2ContextPrecisionMetadata] = None


class EvalResultsUnionMember2ContextRelevancyMetadataRelevantSentence(BaseModel):
    reasons: List[str]

    sentence: str


class EvalResultsUnionMember2ContextRelevancyMetadata(BaseModel):
    relevant_sentences: List[EvalResultsUnionMember2ContextRelevancyMetadataRelevantSentence] = FieldInfo(
        alias="relevantSentences"
    )


class EvalResultsUnionMember2ContextRelevancy(BaseModel):
    name: str

    score: float

    error: Optional[str] = None

    metadata: Optional[EvalResultsUnionMember2ContextRelevancyMetadata] = None


class EvalResultsUnionMember2FaithfulnessMetadataFaithfulness(BaseModel):
    reason: str

    statement: str

    verdict: int

    classification: Optional[Literal["UNSUPPORTED_CLAIM", "CONTRADICTION", "PARTIAL_HALLUCINATION", "SCOPE_DRIFT"]] = (
        None
    )
    """Classification of the hallucination type (only present when verdict is 0)"""


class EvalResultsUnionMember2FaithfulnessMetadata(BaseModel):
    faithfulness: List[EvalResultsUnionMember2FaithfulnessMetadataFaithfulness]

    statements: List[str]


class EvalResultsUnionMember2Faithfulness(BaseModel):
    name: str

    score: float

    error: Optional[str] = None

    metadata: Optional[EvalResultsUnionMember2FaithfulnessMetadata] = None


class EvalResultsUnionMember2(BaseModel):
    answer_relevancy: EvalResultsUnionMember2AnswerRelevancy = FieldInfo(alias="AnswerRelevancy")

    context_precision: EvalResultsUnionMember2ContextPrecision = FieldInfo(alias="ContextPrecision")

    context_relevancy: EvalResultsUnionMember2ContextRelevancy = FieldInfo(alias="ContextRelevancy")

    faithfulness: EvalResultsUnionMember2Faithfulness = FieldInfo(alias="Faithfulness")


class EvalResultsUnionMember3Metadata(BaseModel):
    rationale: str


class EvalResultsUnionMember3(BaseModel):
    name: str

    score: int

    error: Optional[str] = None

    metadata: Optional[EvalResultsUnionMember3Metadata] = None


class EvalResultsUnionMember4Classification(BaseModel):
    fn: List[str] = FieldInfo(alias="FN")
    """False negatives: Statements found in the ground truth but omitted in the answer"""

    fp: List[str] = FieldInfo(alias="FP")
    """
    False positives: Statements present in the answer but not found in the ground
    truth
    """

    tp: List[str] = FieldInfo(alias="TP")
    """
    True positives: Statements that are present in both the answer and the ground
    truth
    """


class EvalResultsUnionMember4(BaseModel):
    classification: EvalResultsUnionMember4Classification

    score: float
    """The F1 score of the response based on the fact checker (0-1)"""

    error: Optional[str] = None

    metadata: Optional[object] = None


class EvalResultsUnionMember5Missing(BaseModel):
    have: float

    need: float

    value: str


class EvalResultsUnionMember5(BaseModel):
    reason: str

    score: float

    expected: Optional[str] = None

    expected_list: Optional[List[str]] = FieldInfo(alias="expectedList", default=None)

    f1: Optional[float] = None

    got: Optional[str] = None

    got_list: Optional[List[str]] = FieldInfo(alias="gotList", default=None)

    match_mode: Optional[Literal["exact_unordered", "contains"]] = FieldInfo(alias="matchMode", default=None)

    missing: Optional[List[EvalResultsUnionMember5Missing]] = None

    precision: Optional[float] = None

    recall: Optional[float] = None

    score_metric: Optional[Literal["f1", "precision", "recall"]] = FieldInfo(alias="scoreMetric", default=None)

    tp: Optional[float] = None


EvalResults: TypeAlias = Union[
    EvalResultsUnionMember0,
    EvalResultsUnionMember1,
    EvalResultsUnionMember2,
    EvalResultsUnionMember3,
    EvalResultsUnionMember4,
    EvalResultsUnionMember5,
]


class Eval(BaseModel):
    """Complete evaluation information"""

    id: str
    """Unique identifier of the evaluation"""

    created_at: datetime = FieldInfo(alias="createdAt")
    """When the evaluation was created"""

    definition: EvalDefinition

    modified_at: datetime = FieldInfo(alias="modifiedAt")
    """When the evaluation was last modified"""

    org_id: str = FieldInfo(alias="orgId")
    """Organization ID that owns this evaluation"""

    status: Literal["PENDING", "IN_PROGRESS", "COMPLETED", "FAILED"]
    """Status of the evaluation/test"""

    passed: Optional[bool] = None
    """Whether the evaluation passed"""

    results: Optional[EvalResults] = None
    """Results of the evaluation (structure depends on eval type)."""

    score: Optional[float] = None
    """Overall score of the evaluation"""


class StepLlmStartStepOutput(BaseModel):
    """Start of an LLM trace."""

    id: str
    """UUID for the step."""

    event: Literal["start"]

    input: str
    """JSON input for this LLM trace event (e.g., the prompt)."""

    api_model_id: str = FieldInfo(alias="modelId")
    """Model ID or name used for the LLM call."""

    status: Literal["success", "error", "timeout", "pending"]
    """Status of the step indicating success or failure. Defaults to 'success'."""

    timestamp: datetime
    """When the step was created"""

    trace_id: str = FieldInfo(alias="traceId")
    """UUID referencing the parent trace's ID."""

    type: Literal["llm"]

    cost_amount: Optional[float] = FieldInfo(alias="costAmount", default=None)
    """Cost of the step in the configured currency (primarily for LLM steps)"""

    duration_ms: Optional[int] = FieldInfo(alias="durationMs", default=None)
    """Duration of the step in milliseconds"""

    end_time: Optional[datetime] = FieldInfo(alias="endTime", default=None)
    """When the step ended (for duration tracking)"""

    error: Optional[str] = None
    """Error message or stack trace if the step failed."""

    group: Optional[str] = None
    """Optionally, the key for a group step to group the step with."""

    metadata: Union[Dict[str, object], List[object], None] = None
    """Extra metadata about this trace event.

    String values are parsed as JSON if possible, otherwise wrapped in { raw: val }.
    """

    name: Optional[str] = None
    """The name of the step."""

    params: Union[Dict[str, object], List[object], None] = None
    """Arbitrary params for the step."""

    parent_id: Optional[str] = FieldInfo(alias="parentId", default=None)
    """Parent step ID for hierarchical span relationships.

    Links this step to its parent in the trace tree.
    """

    status_code: Optional[str] = FieldInfo(alias="statusCode", default=None)
    """
    Status code associated with the step (e.g., HTTP status code for API calls, OTEL
    status code for traced spans).
    """


class StepLlmEndStepOutputUsage(BaseModel):
    """Number of input and output tokens used by the LLM."""

    completion_tokens: int = FieldInfo(alias="completionTokens")
    """Number of completion tokens used by the LLM."""

    prompt_tokens: int = FieldInfo(alias="promptTokens")
    """Number of prompt tokens used by the LLM."""


class StepLlmEndStepOutput(BaseModel):
    """End of an LLM trace."""

    id: str
    """UUID for the step."""

    event: Literal["end"]

    api_model_id: str = FieldInfo(alias="modelId")
    """Model ID or name used for the LLM call."""

    status: Literal["success", "error", "timeout", "pending"]
    """Status of the step indicating success or failure. Defaults to 'success'."""

    timestamp: datetime
    """When the step was created"""

    trace_id: str = FieldInfo(alias="traceId")
    """UUID referencing the parent trace's ID."""

    type: Literal["llm"]

    cost_amount: Optional[float] = FieldInfo(alias="costAmount", default=None)
    """Cost of the step in the configured currency (primarily for LLM steps)"""

    duration_ms: Optional[int] = FieldInfo(alias="durationMs", default=None)
    """Duration of the step in milliseconds"""

    end_time: Optional[datetime] = FieldInfo(alias="endTime", default=None)
    """When the step ended (for duration tracking)"""

    error: Optional[str] = None
    """Error message or stack trace if the step failed."""

    finish_reason: Optional[str] = FieldInfo(alias="finishReason", default=None)
    """
    The reason the LLM stopped generating, extracted from
    gen_ai.response.finish_reasons.
    """

    group: Optional[str] = None
    """Optionally, the key for a group step to group the step with."""

    metadata: Union[Dict[str, object], List[object], None] = None
    """Extra metadata about this trace event.

    String values are parsed as JSON if possible, otherwise wrapped in { raw: val }.
    """

    name: Optional[str] = None
    """The name of the step."""

    output: Optional[str] = None
    """JSON describing the output.

    String inputs are parsed or wrapped in { message: val }.
    """

    params: Union[Dict[str, object], List[object], None] = None
    """Arbitrary params for the step."""

    parent_id: Optional[str] = FieldInfo(alias="parentId", default=None)
    """Parent step ID for hierarchical span relationships.

    Links this step to its parent in the trace tree.
    """

    status_code: Optional[str] = FieldInfo(alias="statusCode", default=None)
    """
    Status code associated with the step (e.g., HTTP status code for API calls, OTEL
    status code for traced spans).
    """

    usage: Optional[StepLlmEndStepOutputUsage] = None
    """Number of input and output tokens used by the LLM."""


class StepToolStepOutput(BaseModel):
    """Track all tool calls using the Tool Step event"""

    id: str
    """UUID for the step."""

    status: Literal["success", "error", "timeout", "pending"]
    """Status of the step indicating success or failure. Defaults to 'success'."""

    timestamp: datetime
    """When the step was created"""

    trace_id: str = FieldInfo(alias="traceId")
    """UUID referencing the parent trace's ID."""

    type: Literal["tool"]

    cost_amount: Optional[float] = FieldInfo(alias="costAmount", default=None)
    """Cost of the step in the configured currency (primarily for LLM steps)"""

    duration_ms: Optional[int] = FieldInfo(alias="durationMs", default=None)
    """Duration of the step in milliseconds"""

    end_time: Optional[datetime] = FieldInfo(alias="endTime", default=None)
    """When the step ended (for duration tracking)"""

    error: Optional[str] = None
    """Error message or stack trace if the step failed."""

    group: Optional[str] = None
    """Optionally, the key for a group step to group the step with."""

    metadata: Union[Dict[str, object], List[object], None] = None
    """Extra metadata about this trace event.

    String values are parsed as JSON if possible, otherwise wrapped in { raw: val }.
    """

    name: Optional[str] = None
    """The name of the step."""

    params: Union[Dict[str, object], List[object], None] = None
    """Arbitrary params for the step."""

    parent_id: Optional[str] = FieldInfo(alias="parentId", default=None)
    """Parent step ID for hierarchical span relationships.

    Links this step to its parent in the trace tree.
    """

    status_code: Optional[str] = FieldInfo(alias="statusCode", default=None)
    """
    Status code associated with the step (e.g., HTTP status code for API calls, OTEL
    status code for traced spans).
    """

    tool_call_id: Optional[str] = FieldInfo(alias="toolCallId", default=None)
    """
    Correlation ID from the LLM's tool_calls response (e.g., OpenAI's call_abc123 or
    Claude's toolu_01A09q). Links tool execution to the originating LLM request in
    parallel tool call scenarios.
    """

    tool_input: Optional[str] = FieldInfo(alias="toolInput", default=None)
    """JSON input for the tool call."""

    tool_output: Optional[str] = FieldInfo(alias="toolOutput", default=None)
    """JSON output from the tool call."""


class StepRetrieverStepOutput(BaseModel):
    """Track all retriever (RAG) calls using the Retriever Step event."""

    id: str
    """UUID for the step."""

    query: str
    """Query used for RAG."""

    result: str
    """Retrieved text"""

    status: Literal["success", "error", "timeout", "pending"]
    """Status of the step indicating success or failure. Defaults to 'success'."""

    timestamp: datetime
    """When the step was created"""

    trace_id: str = FieldInfo(alias="traceId")
    """UUID referencing the parent trace's ID."""

    type: Literal["retriever"]

    cost_amount: Optional[float] = FieldInfo(alias="costAmount", default=None)
    """Cost of the step in the configured currency (primarily for LLM steps)"""

    duration_ms: Optional[int] = FieldInfo(alias="durationMs", default=None)
    """Duration of the step in milliseconds"""

    end_time: Optional[datetime] = FieldInfo(alias="endTime", default=None)
    """When the step ended (for duration tracking)"""

    error: Optional[str] = None
    """Error message or stack trace if the step failed."""

    group: Optional[str] = None
    """Optionally, the key for a group step to group the step with."""

    metadata: Union[Dict[str, object], List[object], None] = None
    """Extra metadata about this trace event.

    String values are parsed as JSON if possible, otherwise wrapped in { raw: val }.
    """

    name: Optional[str] = None
    """The name of the step."""

    params: Union[Dict[str, object], List[object], None] = None
    """Arbitrary params for the step."""

    parent_id: Optional[str] = FieldInfo(alias="parentId", default=None)
    """Parent step ID for hierarchical span relationships.

    Links this step to its parent in the trace tree.
    """

    status_code: Optional[str] = FieldInfo(alias="statusCode", default=None)
    """
    Status code associated with the step (e.g., HTTP status code for API calls, OTEL
    status code for traced spans).
    """


class StepLogStepOutput(BaseModel):
    """Track all logs using the Log Step event."""

    id: str
    """UUID for the step."""

    content: str
    """The actual log message for this trace."""

    status: Literal["success", "error", "timeout", "pending"]
    """Status of the step indicating success or failure. Defaults to 'success'."""

    timestamp: datetime
    """When the step was created"""

    trace_id: str = FieldInfo(alias="traceId")
    """UUID referencing the parent trace's ID."""

    type: Literal["log"]

    cost_amount: Optional[float] = FieldInfo(alias="costAmount", default=None)
    """Cost of the step in the configured currency (primarily for LLM steps)"""

    duration_ms: Optional[int] = FieldInfo(alias="durationMs", default=None)
    """Duration of the step in milliseconds"""

    end_time: Optional[datetime] = FieldInfo(alias="endTime", default=None)
    """When the step ended (for duration tracking)"""

    error: Optional[str] = None
    """Error message or stack trace if the step failed."""

    group: Optional[str] = None
    """Optionally, the key for a group step to group the step with."""

    metadata: Union[Dict[str, object], List[object], None] = None
    """Extra metadata about this trace event.

    String values are parsed as JSON if possible, otherwise wrapped in { raw: val }.
    """

    name: Optional[str] = None
    """The name of the step."""

    params: Union[Dict[str, object], List[object], None] = None
    """Arbitrary params for the step."""

    parent_id: Optional[str] = FieldInfo(alias="parentId", default=None)
    """Parent step ID for hierarchical span relationships.

    Links this step to its parent in the trace tree.
    """

    status_code: Optional[str] = FieldInfo(alias="statusCode", default=None)
    """
    Status code associated with the step (e.g., HTTP status code for API calls, OTEL
    status code for traced spans).
    """


class StepGroupStepOutput(BaseModel):
    """
    Use this to group multiple steps together, for example a log, llm start, and llm end.
    """

    id: str
    """UUID for the step."""

    key: str
    """
    A unique identifier for the grouping, which must be appended to the
    corresponding steps
    """

    status: Literal["success", "error", "timeout", "pending"]
    """Status of the step indicating success or failure. Defaults to 'success'."""

    timestamp: datetime
    """When the step was created"""

    trace_id: str = FieldInfo(alias="traceId")
    """UUID referencing the parent trace's ID."""

    type: Literal["group"]

    cost_amount: Optional[float] = FieldInfo(alias="costAmount", default=None)
    """Cost of the step in the configured currency (primarily for LLM steps)"""

    duration_ms: Optional[int] = FieldInfo(alias="durationMs", default=None)
    """Duration of the step in milliseconds"""

    end_time: Optional[datetime] = FieldInfo(alias="endTime", default=None)
    """When the step ended (for duration tracking)"""

    error: Optional[str] = None
    """Error message or stack trace if the step failed."""

    group: Optional[str] = None
    """Optionally, the key for a group step to group the step with."""

    metadata: Union[Dict[str, object], List[object], None] = None
    """Extra metadata about this trace event.

    String values are parsed as JSON if possible, otherwise wrapped in { raw: val }.
    """

    name: Optional[str] = None
    """The name of the step."""

    params: Union[Dict[str, object], List[object], None] = None
    """Arbitrary params for the step."""

    parent_id: Optional[str] = FieldInfo(alias="parentId", default=None)
    """Parent step ID for hierarchical span relationships.

    Links this step to its parent in the trace tree.
    """

    status_code: Optional[str] = FieldInfo(alias="statusCode", default=None)
    """
    Status code associated with the step (e.g., HTTP status code for API calls, OTEL
    status code for traced spans).
    """


class StepResponseStepOutput(BaseModel):
    """Track AI responses to users using the Response Step event."""

    id: str
    """UUID for the step."""

    content: str
    """The response content from the AI to the user."""

    status: Literal["success", "error", "timeout", "pending"]
    """Status of the step indicating success or failure. Defaults to 'success'."""

    timestamp: datetime
    """When the step was created"""

    trace_id: str = FieldInfo(alias="traceId")
    """UUID referencing the parent trace's ID."""

    type: Literal["response"]

    cost_amount: Optional[float] = FieldInfo(alias="costAmount", default=None)
    """Cost of the step in the configured currency (primarily for LLM steps)"""

    duration_ms: Optional[int] = FieldInfo(alias="durationMs", default=None)
    """Duration of the step in milliseconds"""

    end_time: Optional[datetime] = FieldInfo(alias="endTime", default=None)
    """When the step ended (for duration tracking)"""

    error: Optional[str] = None
    """Error message or stack trace if the step failed."""

    group: Optional[str] = None
    """Optionally, the key for a group step to group the step with."""

    metadata: Union[Dict[str, object], List[object], None] = None
    """Extra metadata about this trace event.

    String values are parsed as JSON if possible, otherwise wrapped in { raw: val }.
    """

    name: Optional[str] = None
    """The name of the step."""

    params: Union[Dict[str, object], List[object], None] = None
    """Arbitrary params for the step."""

    parent_id: Optional[str] = FieldInfo(alias="parentId", default=None)
    """Parent step ID for hierarchical span relationships.

    Links this step to its parent in the trace tree.
    """

    status_code: Optional[str] = FieldInfo(alias="statusCode", default=None)
    """
    Status code associated with the step (e.g., HTTP status code for API calls, OTEL
    status code for traced spans).
    """


class StepRequestStepOutput(BaseModel):
    """Track user requests to the AI using the Request Step event."""

    id: str
    """UUID for the step."""

    content: str
    """The request content from the user to the AI."""

    status: Literal["success", "error", "timeout", "pending"]
    """Status of the step indicating success or failure. Defaults to 'success'."""

    timestamp: datetime
    """When the step was created"""

    trace_id: str = FieldInfo(alias="traceId")
    """UUID referencing the parent trace's ID."""

    type: Literal["request"]

    cost_amount: Optional[float] = FieldInfo(alias="costAmount", default=None)
    """Cost of the step in the configured currency (primarily for LLM steps)"""

    duration_ms: Optional[int] = FieldInfo(alias="durationMs", default=None)
    """Duration of the step in milliseconds"""

    end_time: Optional[datetime] = FieldInfo(alias="endTime", default=None)
    """When the step ended (for duration tracking)"""

    error: Optional[str] = None
    """Error message or stack trace if the step failed."""

    group: Optional[str] = None
    """Optionally, the key for a group step to group the step with."""

    metadata: Union[Dict[str, object], List[object], None] = None
    """Extra metadata about this trace event.

    String values are parsed as JSON if possible, otherwise wrapped in { raw: val }.
    """

    name: Optional[str] = None
    """The name of the step."""

    params: Union[Dict[str, object], List[object], None] = None
    """Arbitrary params for the step."""

    parent_id: Optional[str] = FieldInfo(alias="parentId", default=None)
    """Parent step ID for hierarchical span relationships.

    Links this step to its parent in the trace tree.
    """

    status_code: Optional[str] = FieldInfo(alias="statusCode", default=None)
    """
    Status code associated with the step (e.g., HTTP status code for API calls, OTEL
    status code for traced spans).
    """


Step: TypeAlias = Union[
    StepLlmStartStepOutput,
    StepLlmEndStepOutput,
    StepToolStepOutput,
    StepRetrieverStepOutput,
    StepLogStepOutput,
    StepGroupStepOutput,
    StepResponseStepOutput,
    StepRequestStepOutput,
]


class Trace(BaseModel):
    """A trace grouping related steps (e.g. a user-agent interaction or conversation)."""

    id: str
    """Unique Trace ID (UUID)."""

    timestamp: datetime
    """When the trace was created"""

    evals: Optional[List[Eval]] = None
    """Evaluation results for this trace, joined through the associated test."""

    has_error: Optional[bool] = FieldInfo(alias="hasError", default=None)
    """Whether any step in this trace has an error status."""

    metadata: Union[Dict[str, object], List[object], None] = None
    """Arbitrary metadata (e.g., userId, source).

    String inputs are parsed as JSON or wrapped in { raw: val }.
    """

    reference_id: Optional[str] = FieldInfo(alias="referenceId", default=None)
    """
    An optional reference ID to link the trace to an existing conversation or
    interaction in your own database.
    """

    step_count: Optional[int] = FieldInfo(alias="stepCount", default=None)
    """Total number of steps in this trace."""

    steps: Optional[List[Step]] = None
    """The steps associated with the trace."""

    test_id: Optional[str] = FieldInfo(alias="testId", default=None)
    """The associated Test if this was triggered by an Avido eval"""

    total_completion_tokens: Optional[int] = FieldInfo(alias="totalCompletionTokens", default=None)
    """Total number of completion tokens used across all LLM steps in this trace."""

    total_cost: Optional[float] = FieldInfo(alias="totalCost", default=None)
    """Total cost of all steps in this trace (sum of step costAmount values)."""

    total_duration_ms: Optional[int] = FieldInfo(alias="totalDurationMs", default=None)
    """Total duration of the trace in milliseconds (sum of all step durations)."""

    total_prompt_tokens: Optional[int] = FieldInfo(alias="totalPromptTokens", default=None)
    """Total number of prompt tokens used across all LLM steps in this trace."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import Literal

import httpx

from ..types import topic_list_params, topic_create_params
from .._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..pagination import SyncOffsetPagination, AsyncOffsetPagination
from ..types.topic import Topic
from .._base_client import AsyncPaginator, make_request_options
from ..types.topic_create_response import TopicCreateResponse
from ..types.topic_retrieve_response import TopicRetrieveResponse

__all__ = ["TopicsResource", "AsyncTopicsResource"]


class TopicsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> TopicsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/Avido-AI/avido-py#accessing-raw-response-data-eg-headers
        """
        return TopicsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> TopicsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/Avido-AI/avido-py#with_streaming_response
        """
        return TopicsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        title: str,
        baseline: Optional[float] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TopicCreateResponse:
        """
        Creates a new topic.

        Args:
          title: Title of the topic

          baseline: Optional baseline score for this topic

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/v0/topics",
            body=maybe_transform(
                {
                    "title": title,
                    "baseline": baseline,
                },
                topic_create_params.TopicCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TopicCreateResponse,
        )

    def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TopicRetrieveResponse:
        """
        Retrieves detailed information about a specific topic.

        Args:
          id: The unique identifier of the topic

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/v0/topics/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TopicRetrieveResponse,
        )

    def list(
        self,
        *,
        limit: int | Omit = omit,
        order_by: str | Omit = omit,
        order_dir: Literal["asc", "desc"] | Omit = omit,
        skip: int | Omit = omit,
        title: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncOffsetPagination[Topic]:
        """
        Retrieves a paginated list of topics with optional filtering.

        Args:
          limit: Number of items to include in the result set.

          order_by: Field to order by in the result set.

          order_dir: Order direction.

          skip: Number of items to skip before starting to collect the result set.

          title: Filter by topic title (case-insensitive)

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/v0/topics",
            page=SyncOffsetPagination[Topic],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "limit": limit,
                        "order_by": order_by,
                        "order_dir": order_dir,
                        "skip": skip,
                        "title": title,
                    },
                    topic_list_params.TopicListParams,
                ),
            ),
            model=Topic,
        )


class AsyncTopicsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncTopicsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/Avido-AI/avido-py#accessing-raw-response-data-eg-headers
        """
        return AsyncTopicsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncTopicsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/Avido-AI/avido-py#with_streaming_response
        """
        return AsyncTopicsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        title: str,
        baseline: Optional[float] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TopicCreateResponse:
        """
        Creates a new topic.

        Args:
          title: Title of the topic

          baseline: Optional baseline score for this topic

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/v0/topics",
            body=await async_maybe_transform(
                {
                    "title": title,
                    "baseline": baseline,
                },
                topic_create_params.TopicCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TopicCreateResponse,
        )

    async def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TopicRetrieveResponse:
        """
        Retrieves detailed information about a specific topic.

        Args:
          id: The unique identifier of the topic

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/v0/topics/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TopicRetrieveResponse,
        )

    def list(
        self,
        *,
        limit: int | Omit = omit,
        order_by: str | Omit = omit,
        order_dir: Literal["asc", "desc"] | Omit = omit,
        skip: int | Omit = omit,
        title: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[Topic, AsyncOffsetPagination[Topic]]:
        """
        Retrieves a paginated list of topics with optional filtering.

        Args:
          limit: Number of items to include in the result set.

          order_by: Field to order by in the result set.

          order_dir: Order direction.

          skip: Number of items to skip before starting to collect the result set.

          title: Filter by topic title (case-insensitive)

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/v0/topics",
            page=AsyncOffsetPagination[Topic],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "limit": limit,
                        "order_by": order_by,
                        "order_dir": order_dir,
                        "skip": skip,
                        "title": title,
                    },
                    topic_list_params.TopicListParams,
                ),
            ),
            model=Topic,
        )


class TopicsResourceWithRawResponse:
    def __init__(self, topics: TopicsResource) -> None:
        self._topics = topics

        self.create = to_raw_response_wrapper(
            topics.create,
        )
        self.retrieve = to_raw_response_wrapper(
            topics.retrieve,
        )
        self.list = to_raw_response_wrapper(
            topics.list,
        )


class AsyncTopicsResourceWithRawResponse:
    def __init__(self, topics: AsyncTopicsResource) -> None:
        self._topics = topics

        self.create = async_to_raw_response_wrapper(
            topics.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            topics.retrieve,
        )
        self.list = async_to_raw_response_wrapper(
            topics.list,
        )


class TopicsResourceWithStreamingResponse:
    def __init__(self, topics: TopicsResource) -> None:
        self._topics = topics

        self.create = to_streamed_response_wrapper(
            topics.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            topics.retrieve,
        )
        self.list = to_streamed_response_wrapper(
            topics.list,
        )


class AsyncTopicsResourceWithStreamingResponse:
    def __init__(self, topics: AsyncTopicsResource) -> None:
        self._topics = topics

        self.create = async_to_streamed_response_wrapper(
            topics.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            topics.retrieve,
        )
        self.list = async_to_streamed_response_wrapper(
            topics.list,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from datetime import datetime
from typing_extensions import Literal

import httpx

from ..types import trace_list_params
from .._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from .._utils import maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..pagination import SyncOffsetPagination, AsyncOffsetPagination
from ..types.trace import Trace
from .._base_client import AsyncPaginator, make_request_options
from ..types.trace_retrieve_response import TraceRetrieveResponse
from ..types.trace_retrieve_by_test_response import TraceRetrieveByTestResponse

__all__ = ["TracesResource", "AsyncTracesResource"]


class TracesResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> TracesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/Avido-AI/avido-py#accessing-raw-response-data-eg-headers
        """
        return TracesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> TracesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/Avido-AI/avido-py#with_streaming_response
        """
        return TracesResourceWithStreamingResponse(self)

    def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TraceRetrieveResponse:
        """
        Retrieves detailed information about a specific trace.

        Args:
          id: The unique identifier of the trace

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/v0/traces/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TraceRetrieveResponse,
        )

    def list(
        self,
        *,
        end_date: Union[str, datetime] | Omit = omit,
        has_error: bool | Omit = omit,
        limit: int | Omit = omit,
        max_cost: float | Omit = omit,
        max_duration: int | Omit = omit,
        max_score: float | Omit = omit,
        metadata_key: str | Omit = omit,
        metadata_value: str | Omit = omit,
        min_cost: float | Omit = omit,
        min_duration: int | Omit = omit,
        min_score: float | Omit = omit,
        order_by: str | Omit = omit,
        order_dir: Literal["asc", "desc"] | Omit = omit,
        reference_id: str | Omit = omit,
        search: str | Omit = omit,
        skip: int | Omit = omit,
        start_date: Union[str, datetime] | Omit = omit,
        test_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncOffsetPagination[Trace]:
        """
        Retrieve traces with associated steps, filtered by application ID and optional
        date parameters.

        Args:
          end_date: Filter traces created before this date (inclusive).

          has_error: Filter traces with or without errors.

          limit: Number of items to include in the result set.

          max_cost: Filter traces with total cost less than or equal to this value.

          max_duration: Filter traces with total duration (ms) less than or equal to this value.

          max_score: Filter traces with eval score less than or equal to this value.

          metadata_key: Metadata key to filter on. Must be used together with metadataValue.

          metadata_value: Metadata value to filter on. Must be used together with metadataKey.

          min_cost: Filter traces with total cost greater than or equal to this value.

          min_duration: Filter traces with total duration (ms) greater than or equal to this value.

          min_score: Filter traces with eval score greater than or equal to this value.

          order_by: Field to order by in the result set.

          order_dir: Order direction.

          reference_id: Filter traces by reference ID

          search: Full-text search across trace step content fields (input, output, toolInput,
              toolOutput, content, query, result).

          skip: Number of items to skip before starting to collect the result set.

          start_date: Filter traces created after this date (inclusive).

          test_id: Filter traces by test ID

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/v0/traces",
            page=SyncOffsetPagination[Trace],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "end_date": end_date,
                        "has_error": has_error,
                        "limit": limit,
                        "max_cost": max_cost,
                        "max_duration": max_duration,
                        "max_score": max_score,
                        "metadata_key": metadata_key,
                        "metadata_value": metadata_value,
                        "min_cost": min_cost,
                        "min_duration": min_duration,
                        "min_score": min_score,
                        "order_by": order_by,
                        "order_dir": order_dir,
                        "reference_id": reference_id,
                        "search": search,
                        "skip": skip,
                        "start_date": start_date,
                        "test_id": test_id,
                    },
                    trace_list_params.TraceListParams,
                ),
            ),
            model=Trace,
        )

    def retrieve_by_test(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TraceRetrieveByTestResponse:
        """
        Retrieves detailed information about a specific trace.

        Args:
          id: The unique identifier of the trace

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/v0/traces/by-test/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TraceRetrieveByTestResponse,
        )


class AsyncTracesResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncTracesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/Avido-AI/avido-py#accessing-raw-response-data-eg-headers
        """
        return AsyncTracesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncTracesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/Avido-AI/avido-py#with_streaming_response
        """
        return AsyncTracesResourceWithStreamingResponse(self)

    async def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TraceRetrieveResponse:
        """
        Retrieves detailed information about a specific trace.

        Args:
          id: The unique identifier of the trace

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/v0/traces/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TraceRetrieveResponse,
        )

    def list(
        self,
        *,
        end_date: Union[str, datetime] | Omit = omit,
        has_error: bool | Omit = omit,
        limit: int | Omit = omit,
        max_cost: float | Omit = omit,
        max_duration: int | Omit = omit,
        max_score: float | Omit = omit,
        metadata_key: str | Omit = omit,
        metadata_value: str | Omit = omit,
        min_cost: float | Omit = omit,
        min_duration: int | Omit = omit,
        min_score: float | Omit = omit,
        order_by: str | Omit = omit,
        order_dir: Literal["asc", "desc"] | Omit = omit,
        reference_id: str | Omit = omit,
        search: str | Omit = omit,
        skip: int | Omit = omit,
        start_date: Union[str, datetime] | Omit = omit,
        test_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[Trace, AsyncOffsetPagination[Trace]]:
        """
        Retrieve traces with associated steps, filtered by application ID and optional
        date parameters.

        Args:
          end_date: Filter traces created before this date (inclusive).

          has_error: Filter traces with or without errors.

          limit: Number of items to include in the result set.

          max_cost: Filter traces with total cost less than or equal to this value.

          max_duration: Filter traces with total duration (ms) less than or equal to this value.

          max_score: Filter traces with eval score less than or equal to this value.

          metadata_key: Metadata key to filter on. Must be used together with metadataValue.

          metadata_value: Metadata value to filter on. Must be used together with metadataKey.

          min_cost: Filter traces with total cost greater than or equal to this value.

          min_duration: Filter traces with total duration (ms) greater than or equal to this value.

          min_score: Filter traces with eval score greater than or equal to this value.

          order_by: Field to order by in the result set.

          order_dir: Order direction.

          reference_id: Filter traces by reference ID

          search: Full-text search across trace step content fields (input, output, toolInput,
              toolOutput, content, query, result).

          skip: Number of items to skip before starting to collect the result set.

          start_date: Filter traces created after this date (inclusive).

          test_id: Filter traces by test ID

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/v0/traces",
            page=AsyncOffsetPagination[Trace],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "end_date": end_date,
                        "has_error": has_error,
                        "limit": limit,
                        "max_cost": max_cost,
                        "max_duration": max_duration,
                        "max_score": max_score,
                        "metadata_key": metadata_key,
                        "metadata_value": metadata_value,
                        "min_cost": min_cost,
                        "min_duration": min_duration,
                        "min_score": min_score,
                        "order_by": order_by,
                        "order_dir": order_dir,
                        "reference_id": reference_id,
                        "search": search,
                        "skip": skip,
                        "start_date": start_date,
                        "test_id": test_id,
                    },
                    trace_list_params.TraceListParams,
                ),
            ),
            model=Trace,
        )

    async def retrieve_by_test(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TraceRetrieveByTestResponse:
        """
        Retrieves detailed information about a specific trace.

        Args:
          id: The unique identifier of the trace

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/v0/traces/by-test/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TraceRetrieveByTestResponse,
        )


class TracesResourceWithRawResponse:
    def __init__(self, traces: TracesResource) -> None:
        self._traces = traces

        self.retrieve = to_raw_response_wrapper(
            traces.retrieve,
        )
        self.list = to_raw_response_wrapper(
            traces.list,
        )
        self.retrieve_by_test = to_raw_response_wrapper(
            traces.retrieve_by_test,
        )


class AsyncTracesResourceWithRawResponse:
    def __init__(self, traces: AsyncTracesResource) -> None:
        self._traces = traces

        self.retrieve = async_to_raw_response_wrapper(
            traces.retrieve,
        )
        self.list = async_to_raw_response_wrapper(
            traces.list,
        )
        self.retrieve_by_test = async_to_raw_response_wrapper(
            traces.retrieve_by_test,
        )


class TracesResourceWithStreamingResponse:
    def __init__(self, traces: TracesResource) -> None:
        self._traces = traces

        self.retrieve = to_streamed_response_wrapper(
            traces.retrieve,
        )
        self.list = to_streamed_response_wrapper(
            traces.list,
        )
        self.retrieve_by_test = to_streamed_response_wrapper(
            traces.retrieve_by_test,
        )


class AsyncTracesResourceWithStreamingResponse:
    def __init__(self, traces: AsyncTracesResource) -> None:
        self._traces = traces

        self.retrieve = async_to_streamed_response_wrapper(
            traces.retrieve,
        )
        self.list = async_to_streamed_response_wrapper(
            traces.list,
        )
        self.retrieve_by_test = async_to_streamed_response_wrapper(
            traces.retrieve_by_test,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing_extensions import Literal, TypeAlias

__all__ = ["EvalType"]

EvalType: TypeAlias = Literal["NATURALNESS", "STYLE", "RECALL", "CUSTOM", "FACT", "OUTPUT_MATCH"]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["DefinitionListParams"]


class DefinitionListParams(TypedDict, total=False):
    limit: int
    """Number of items to include in the result set."""

    order_by: Annotated[str, PropertyInfo(alias="orderBy")]
    """Field to order by in the result set."""

    order_dir: Annotated[Literal["asc", "desc"], PropertyInfo(alias="orderDir")]
    """Order direction."""

    skip: int
    """Number of items to skip before starting to collect the result set."""

    status: Literal["PENDING", "IN_PROGRESS", "COMPLETED", "FAILED"]
    """Filter by evaluation status (e.g. COMPLETED)"""

    task_id: Annotated[str, PropertyInfo(alias="taskId")]
    """Filter by task ID"""

    trace_id: Annotated[str, PropertyInfo(alias="traceId")]
    """Filter by trace ID"""

    type: str
    """Filter by evaluation type (e.g. NATURALNESS)"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from .._models import BaseModel

__all__ = ["IngestCreateResponse", "Data"]


class Data(BaseModel):
    success: bool
    """Whether the event(s) were successfully ingested."""

    id: Optional[str] = None
    """Trace ID if the event was a trace. Not returned for steps."""

    error: Optional[str] = None
    """Error message if ingestion failed."""


class IngestCreateResponse(BaseModel):
    """Response schema for successful event ingestion."""

    data: List[Data]
    """Array of results for each ingested event."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing_extensions import Literal, TypeAlias

__all__ = ["RunType"]

RunType: TypeAlias = Literal["MANUAL", "SCHEDULED", "EXPERIMENT", "MONITORING"]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import TYPE_CHECKING, Any, Mapping
from typing_extensions import Self, override

import httpx

from . import _exceptions
from ._qs import Querystring
from ._types import (
    Omit,
    Timeout,
    NotGiven,
    Transport,
    ProxiesTypes,
    RequestOptions,
    not_given,
)
from ._utils import is_given, get_async_library
from ._compat import cached_property
from ._models import SecurityOptions
from ._version import __version__
from ._streaming import Stream as Stream, AsyncStream as AsyncStream
from ._exceptions import AvidoError, APIStatusError
from ._base_client import (
    DEFAULT_MAX_RETRIES,
    SyncAPIClient,
    AsyncAPIClient,
)

if TYPE_CHECKING:
    from .resources import (
        tags,
        tasks,
        tests,
        ingest,
        issues,
        topics,
        traces,
        documents,
        definitions,
        applications,
        style_guides,
        validate_webhook,
    )
    from .resources.tags import TagsResource, AsyncTagsResource
    from .resources.tests import TestsResource, AsyncTestsResource
    from .resources.ingest import IngestResource, AsyncIngestResource
    from .resources.issues import IssuesResource, AsyncIssuesResource
    from .resources.topics import TopicsResource, AsyncTopicsResource
    from .resources.traces import TracesResource, AsyncTracesResource
    from .resources.definitions import DefinitionsResource, AsyncDefinitionsResource
    from .resources.tasks.tasks import TasksResource, AsyncTasksResource
    from .resources.applications import ApplicationsResource, AsyncApplicationsResource
    from .resources.style_guides import StyleGuidesResource, AsyncStyleGuidesResource
    from .resources.validate_webhook import ValidateWebhookResource, AsyncValidateWebhookResource
    from .resources.documents.documents import DocumentsResource, AsyncDocumentsResource

__all__ = ["Timeout", "Transport", "ProxiesTypes", "RequestOptions", "Avido", "AsyncAvido", "Client", "AsyncClient"]


class Avido(SyncAPIClient):
    # client options
    api_key: str
    application_id: str

    def __init__(
        self,
        *,
        api_key: str | None = None,
        application_id: str | None = None,
        base_url: str | httpx.URL | None = None,
        timeout: float | Timeout | None | NotGiven = not_given,
        max_retries: int = DEFAULT_MAX_RETRIES,
        default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        # Configure a custom httpx client.
        # We provide a `DefaultHttpxClient` class that you can pass to retain the default values we use for `limits`, `timeout` & `follow_redirects`.
        # See the [httpx documentation](https://www.python-httpx.org/api/#client) for more details.
        http_client: httpx.Client | None = None,
        # Enable or disable schema validation for data returned by the API.
        # When enabled an error APIResponseValidationError is raised
        # if the API responds with invalid data for the expected schema.
        #
        # This parameter may be removed or changed in the future.
        # If you rely on this feature, please open a GitHub issue
        # outlining your use-case to help us decide if it should be
        # part of our public interface in the future.
        _strict_response_validation: bool = False,
    ) -> None:
        """Construct a new synchronous Avido client instance.

        This automatically infers the following arguments from their corresponding environment variables if they are not provided:
        - `api_key` from `AVIDO_API_KEY`
        - `application_id` from `AVIDO_APPLICATION_ID`
        """
        if api_key is None:
            api_key = os.environ.get("AVIDO_API_KEY")
        if api_key is None:
            raise AvidoError(
                "The api_key client option must be set either by passing api_key to the client or by setting the AVIDO_API_KEY environment variable"
            )
        self.api_key = api_key

        if application_id is None:
            application_id = os.environ.get("AVIDO_APPLICATION_ID")
        if application_id is None:
            raise AvidoError(
                "The application_id client option must be set either by passing application_id to the client or by setting the AVIDO_APPLICATION_ID environment variable"
            )
        self.application_id = application_id

        if base_url is None:
            base_url = os.environ.get("AVIDO_BASE_URL")
        if base_url is None:
            base_url = f"https://api.avidoai.com"

        super().__init__(
            version=__version__,
            base_url=base_url,
            max_retries=max_retries,
            timeout=timeout,
            http_client=http_client,
            custom_headers=default_headers,
            custom_query=default_query,
            _strict_response_validation=_strict_response_validation,
        )

    @cached_property
    def validate_webhook(self) -> ValidateWebhookResource:
        from .resources.validate_webhook import ValidateWebhookResource

        return ValidateWebhookResource(self)

    @cached_property
    def applications(self) -> ApplicationsResource:
        from .resources.applications import ApplicationsResource

        return ApplicationsResource(self)

    @cached_property
    def traces(self) -> TracesResource:
        from .resources.traces import TracesResource

        return TracesResource(self)

    @cached_property
    def ingest(self) -> IngestResource:
        from .resources.ingest import IngestResource

        return IngestResource(self)

    @cached_property
    def tasks(self) -> TasksResource:
        from .resources.tasks import TasksResource

        return TasksResource(self)

    @cached_property
    def definitions(self) -> DefinitionsResource:
        from .resources.definitions import DefinitionsResource

        return DefinitionsResource(self)

    @cached_property
    def tests(self) -> TestsResource:
        from .resources.tests import TestsResource

        return TestsResource(self)

    @cached_property
    def topics(self) -> TopicsResource:
        from .resources.topics import TopicsResource

        return TopicsResource(self)

    @cached_property
    def issues(self) -> IssuesResource:
        from .resources.issues import IssuesResource

        return IssuesResource(self)

    @cached_property
    def style_guides(self) -> StyleGuidesResource:
        from .resources.style_guides import StyleGuidesResource

        return StyleGuidesResource(self)

    @cached_property
    def documents(self) -> DocumentsResource:
        from .resources.documents import DocumentsResource

        return DocumentsResource(self)

    @cached_property
    def tags(self) -> TagsResource:
        from .resources.tags import TagsResource

        return TagsResource(self)

    @cached_property
    def with_raw_response(self) -> AvidoWithRawResponse:
        return AvidoWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AvidoWithStreamedResponse:
        return AvidoWithStreamedResponse(self)

    @property
    @override
    def qs(self) -> Querystring:
        return Querystring(array_format="comma")

    @override
    def _auth_headers(self, security: SecurityOptions) -> dict[str, str]:
        return {
            **(self._api_key if security.get("api_key", False) else {}),
            **(self._application_id if security.get("application_id", False) else {}),
        }

    @property
    def _api_key(self) -> dict[str, str]:
        api_key = self.api_key
        return {"x-api-key": api_key}

    @property
    def _application_id(self) -> dict[str, str]:
        application_id = self.application_id
        return {"x-application-id": application_id}

    @property
    @override
    def default_headers(self) -> dict[str, str | Omit]:
        return {
            **super().default_headers,
            "X-Stainless-Async": "false",
            **self._custom_headers,
        }

    def copy(
        self,
        *,
        api_key: str | None = None,
        application_id: str | None = None,
        base_url: str | httpx.URL | None = None,
        timeout: float | Timeout | None | NotGiven = not_given,
        http_client: httpx.Client | None = None,
        max_retries: int | NotGiven = not_given,
        default_headers: Mapping[str, str] | None = None,
        set_default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        set_default_query: Mapping[str, object] | None = None,
        _extra_kwargs: Mapping[str, Any] = {},
    ) -> Self:
        """
        Create a new client instance re-using the same options given to the current client with optional overriding.
        """
        if default_headers is not None and set_default_headers is not None:
            raise ValueError("The `default_headers` and `set_default_headers` arguments are mutually exclusive")

        if default_query is not None and set_default_query is not None:
            raise ValueError("The `default_query` and `set_default_query` arguments are mutually exclusive")

        headers = self._custom_headers
        if default_headers is not None:
            headers = {**headers, **default_headers}
        elif set_default_headers is not None:
            headers = set_default_headers

        params = self._custom_query
        if default_query is not None:
            params = {**params, **default_query}
        elif set_default_query is not None:
            params = set_default_query

        http_client = http_client or self._client
        return self.__class__(
            api_key=api_key or self.api_key,
            application_id=application_id or self.application_id,
            base_url=base_url or self.base_url,
            timeout=self.timeout if isinstance(timeout, NotGiven) else timeout,
            http_client=http_client,
            max_retries=max_retries if is_given(max_retries) else self.max_retries,
            default_headers=headers,
            default_query=params,
            **_extra_kwargs,
        )

    # Alias for `copy` for nicer inline usage, e.g.
    # client.with_options(timeout=10).foo.create(...)
    with_options = copy

    @override
    def _make_status_error(
        self,
        err_msg: str,
        *,
        body: object,
        response: httpx.Response,
    ) -> APIStatusError:
        if response.status_code == 400:
            return _exceptions.BadRequestError(err_msg, response=response, body=body)

        if response.status_code == 401:
            return _exceptions.AuthenticationError(err_msg, response=response, body=body)

        if response.status_code == 403:
            return _exceptions.PermissionDeniedError(err_msg, response=response, body=body)

        if response.status_code == 404:
            return _exceptions.NotFoundError(err_msg, response=response, body=body)

        if response.status_code == 409:
            return _exceptions.ConflictError(err_msg, response=response, body=body)

        if response.status_code == 422:
            return _exceptions.UnprocessableEntityError(err_msg, response=response, body=body)

        if response.status_code == 429:
            return _exceptions.RateLimitError(err_msg, response=response, body=body)

        if response.status_code >= 500:
            return _exceptions.InternalServerError(err_msg, response=response, body=body)
        return APIStatusError(err_msg, response=response, body=body)


class AsyncAvido(AsyncAPIClient):
    # client options
    api_key: str
    application_id: str

    def __init__(
        self,
        *,
        api_key: str | None = None,
        application_id: str | None = None,
        base_url: str | httpx.URL | None = None,
        timeout: float | Timeout | None | NotGiven = not_given,
        max_retries: int = DEFAULT_MAX_RETRIES,
        default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        # Configure a custom httpx client.
        # We provide a `DefaultAsyncHttpxClient` class that you can pass to retain the default values we use for `limits`, `timeout` & `follow_redirects`.
        # See the [httpx documentation](https://www.python-httpx.org/api/#asyncclient) for more details.
        http_client: httpx.AsyncClient | None = None,
        # Enable or disable schema validation for data returned by the API.
        # When enabled an error APIResponseValidationError is raised
        # if the API responds with invalid data for the expected schema.
        #
        # This parameter may be removed or changed in the future.
        # If you rely on this feature, please open a GitHub issue
        # outlining your use-case to help us decide if it should be
        # part of our public interface in the future.
        _strict_response_validation: bool = False,
    ) -> None:
        """Construct a new async AsyncAvido client instance.

        This automatically infers the following arguments from their corresponding environment variables if they are not provided:
        - `api_key` from `AVIDO_API_KEY`
        - `application_id` from `AVIDO_APPLICATION_ID`
        """
        if api_key is None:
            api_key = os.environ.get("AVIDO_API_KEY")
        if api_key is None:
            raise AvidoError(
                "The api_key client option must be set either by passing api_key to the client or by setting the AVIDO_API_KEY environment variable"
            )
        self.api_key = api_key

        if application_id is None:
            application_id = os.environ.get("AVIDO_APPLICATION_ID")
        if application_id is None:
            raise AvidoError(
                "The application_id client option must be set either by passing application_id to the client or by setting the AVIDO_APPLICATION_ID environment variable"
            )
        self.application_id = application_id

        if base_url is None:
            base_url = os.environ.get("AVIDO_BASE_URL")
        if base_url is None:
            base_url = f"https://api.avidoai.com"

        super().__init__(
            version=__version__,
            base_url=base_url,
            max_retries=max_retries,
            timeout=timeout,
            http_client=http_client,
            custom_headers=default_headers,
            custom_query=default_query,
            _strict_response_validation=_strict_response_validation,
        )

    @cached_property
    def validate_webhook(self) -> AsyncValidateWebhookResource:
        from .resources.validate_webhook import AsyncValidateWebhookResource

        return AsyncValidateWebhookResource(self)

    @cached_property
    def applications(self) -> AsyncApplicationsResource:
        from .resources.applications import AsyncApplicationsResource

        return AsyncApplicationsResource(self)

    @cached_property
    def traces(self) -> AsyncTracesResource:
        from .resources.traces import AsyncTracesResource

        return AsyncTracesResource(self)

    @cached_property
    def ingest(self) -> AsyncIngestResource:
        from .resources.ingest import AsyncIngestResource

        return AsyncIngestResource(self)

    @cached_property
    def tasks(self) -> AsyncTasksResource:
        from .resources.tasks import AsyncTasksResource

        return AsyncTasksResource(self)

    @cached_property
    def definitions(self) -> AsyncDefinitionsResource:
        from .resources.definitions import AsyncDefinitionsResource

        return AsyncDefinitionsResource(self)

    @cached_property
    def tests(self) -> AsyncTestsResource:
        from .resources.tests import AsyncTestsResource

        return AsyncTestsResource(self)

    @cached_property
    def topics(self) -> AsyncTopicsResource:
        from .resources.topics import AsyncTopicsResource

        return AsyncTopicsResource(self)

    @cached_property
    def issues(self) -> AsyncIssuesResource:
        from .resources.issues import AsyncIssuesResource

        return AsyncIssuesResource(self)

    @cached_property
    def style_guides(self) -> AsyncStyleGuidesResource:
        from .resources.style_guides import AsyncStyleGuidesResource

        return AsyncStyleGuidesResource(self)

    @cached_property
    def documents(self) -> AsyncDocumentsResource:
        from .resources.documents import AsyncDocumentsResource

        return AsyncDocumentsResource(self)

    @cached_property
    def tags(self) -> AsyncTagsResource:
        from .resources.tags import AsyncTagsResource

        return AsyncTagsResource(self)

    @cached_property
    def with_raw_response(self) -> AsyncAvidoWithRawResponse:
        return AsyncAvidoWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncAvidoWithStreamedResponse:
        return AsyncAvidoWithStreamedResponse(self)

    @property
    @override
    def qs(self) -> Querystring:
        return Querystring(array_format="comma")

    @override
    def _auth_headers(self, security: SecurityOptions) -> dict[str, str]:
        return {
            **(self._api_key if security.get("api_key", False) else {}),
            **(self._application_id if security.get("application_id", False) else {}),
        }

    @property
    def _api_key(self) -> dict[str, str]:
        api_key = self.api_key
        return {"x-api-key": api_key}

    @property
    def _application_id(self) -> dict[str, str]:
        application_id = self.application_id
        return {"x-application-id": application_id}

    @property
    @override
    def default_headers(self) -> dict[str, str | Omit]:
        return {
            **super().default_headers,
            "X-Stainless-Async": f"async:{get_async_library()}",
            **self._custom_headers,
        }

    def copy(
        self,
        *,
        api_key: str | None = None,
        application_id: str | None = None,
        base_url: str | httpx.URL | None = None,
        timeout: float | Timeout | None | NotGiven = not_given,
        http_client: httpx.AsyncClient | None = None,
        max_retries: int | NotGiven = not_given,
        default_headers: Mapping[str, str] | None = None,
        set_default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        set_default_query: Mapping[str, object] | None = None,
        _extra_kwargs: Mapping[str, Any] = {},
    ) -> Self:
        """
        Create a new client instance re-using the same options given to the current client with optional overriding.
        """
        if default_headers is not None and set_default_headers is not None:
            raise ValueError("The `default_headers` and `set_default_headers` arguments are mutually exclusive")

        if default_query is not None and set_default_query is not None:
            raise ValueError("The `default_query` and `set_default_query` arguments are mutually exclusive")

        headers = self._custom_headers
        if default_headers is not None:
            headers = {**headers, **default_headers}
        elif set_default_headers is not None:
            headers = set_default_headers

        params = self._custom_query
        if default_query is not None:
            params = {**params, **default_query}
        elif set_default_query is not None:
            params = set_default_query

        http_client = http_client or self._client
        return self.__class__(
            api_key=api_key or self.api_key,
            application_id=application_id or self.application_id,
            base_url=base_url or self.base_url,
            timeout=self.timeout if isinstance(timeout, NotGiven) else timeout,
            http_client=http_client,
            max_retries=max_retries if is_given(max_retries) else self.max_retries,
            default_headers=headers,
            default_query=params,
            **_extra_kwargs,
        )

    # Alias for `copy` for nicer inline usage, e.g.
    # client.with_options(timeout=10).foo.create(...)
    with_options = copy

    @override
    def _make_status_error(
        self,
        err_msg: str,
        *,
        body: object,
        response: httpx.Response,
    ) -> APIStatusError:
        if response.status_code == 400:
            return _exceptions.BadRequestError(err_msg, response=response, body=body)

        if response.status_code == 401:
            return _exceptions.AuthenticationError(err_msg, response=response, body=body)

        if response.status_code == 403:
            return _exceptions.PermissionDeniedError(err_msg, response=response, body=body)

        if response.status_code == 404:
            return _exceptions.NotFoundError(err_msg, response=response, body=body)

        if response.status_code == 409:
            return _exceptions.ConflictError(err_msg, response=response, body=body)

        if response.status_code == 422:
            return _exceptions.UnprocessableEntityError(err_msg, response=response, body=body)

        if response.status_code == 429:
            return _exceptions.RateLimitError(err_msg, response=response, body=body)

        if response.status_code >= 500:
            return _exceptions.InternalServerError(err_msg, response=response, body=body)
        return APIStatusError(err_msg, response=response, body=body)


class AvidoWithRawResponse:
    _client: Avido

    def __init__(self, client: Avido) -> None:
        self._client = client

    @cached_property
    def validate_webhook(self) -> validate_webhook.ValidateWebhookResourceWithRawResponse:
        from .resources.validate_webhook import ValidateWebhookResourceWithRawResponse

        return ValidateWebhookResourceWithRawResponse(self._client.validate_webhook)

    @cached_property
    def applications(self) -> applications.ApplicationsResourceWithRawResponse:
        from .resources.applications import ApplicationsResourceWithRawResponse

        return ApplicationsResourceWithRawResponse(self._client.applications)

    @cached_property
    def traces(self) -> traces.TracesResourceWithRawResponse:
        from .resources.traces import TracesResourceWithRawResponse

        return TracesResourceWithRawResponse(self._client.traces)

    @cached_property
    def ingest(self) -> ingest.IngestResourceWithRawResponse:
        from .resources.ingest import IngestResourceWithRawResponse

        return IngestResourceWithRawResponse(self._client.ingest)

    @cached_property
    def tasks(self) -> tasks.TasksResourceWithRawResponse:
        from .resources.tasks import TasksResourceWithRawResponse

        return TasksResourceWithRawResponse(self._client.tasks)

    @cached_property
    def definitions(self) -> definitions.DefinitionsResourceWithRawResponse:
        from .resources.definitions import DefinitionsResourceWithRawResponse

        return DefinitionsResourceWithRawResponse(self._client.definitions)

    @cached_property
    def tests(self) -> tests.TestsResourceWithRawResponse:
        from .resources.tests import TestsResourceWithRawResponse

        return TestsResourceWithRawResponse(self._client.tests)

    @cached_property
    def topics(self) -> topics.TopicsResourceWithRawResponse:
        from .resources.topics import TopicsResourceWithRawResponse

        return TopicsResourceWithRawResponse(self._client.topics)

    @cached_property
    def issues(self) -> issues.IssuesResourceWithRawResponse:
        from .resources.issues import IssuesResourceWithRawResponse

        return IssuesResourceWithRawResponse(self._client.issues)

    @cached_property
    def style_guides(self) -> style_guides.StyleGuidesResourceWithRawResponse:
        from .resources.style_guides import StyleGuidesResourceWithRawResponse

        return StyleGuidesResourceWithRawResponse(self._client.style_guides)

    @cached_property
    def documents(self) -> documents.DocumentsResourceWithRawResponse:
        from .resources.documents import DocumentsResourceWithRawResponse

        return DocumentsResourceWithRawResponse(self._client.documents)

    @cached_property
    def tags(self) -> tags.TagsResourceWithRawResponse:
        from .resources.tags import TagsResourceWithRawResponse

        return TagsResourceWithRawResponse(self._client.tags)


class AsyncAvidoWithRawResponse:
    _client: AsyncAvido

    def __init__(self, client: AsyncAvido) -> None:
        self._client = client

    @cached_property
    def validate_webhook(self) -> validate_webhook.AsyncValidateWebhookResourceWithRawResponse:
        from .resources.validate_webhook import AsyncValidateWebhookResourceWithRawResponse

        return AsyncValidateWebhookResourceWithRawResponse(self._client.validate_webhook)

    @cached_property
    def applications(self) -> applications.AsyncApplicationsResourceWithRawResponse:
        from .resources.applications import AsyncApplicationsResourceWithRawResponse

        return AsyncApplicationsResourceWithRawResponse(self._client.applications)

    @cached_property
    def traces(self) -> traces.AsyncTracesResourceWithRawResponse:
        from .resources.traces import AsyncTracesResourceWithRawResponse

        return AsyncTracesResourceWithRawResponse(self._client.traces)

    @cached_property
    def ingest(self) -> ingest.AsyncIngestResourceWithRawResponse:
        from .resources.ingest import AsyncIngestResourceWithRawResponse

        return AsyncIngestResourceWithRawResponse(self._client.ingest)

    @cached_property
    def tasks(self) -> tasks.AsyncTasksResourceWithRawResponse:
        from .resources.tasks import AsyncTasksResourceWithRawResponse

        return AsyncTasksResourceWithRawResponse(self._client.tasks)

    @cached_property
    def definitions(self) -> definitions.AsyncDefinitionsResourceWithRawResponse:
        from .resources.definitions import AsyncDefinitionsResourceWithRawResponse

        return AsyncDefinitionsResourceWithRawResponse(self._client.definitions)

    @cached_property
    def tests(self) -> tests.AsyncTestsResourceWithRawResponse:
        from .resources.tests import AsyncTestsResourceWithRawResponse

        return AsyncTestsResourceWithRawResponse(self._client.tests)

    @cached_property
    def topics(self) -> topics.AsyncTopicsResourceWithRawResponse:
        from .resources.topics import AsyncTopicsResourceWithRawResponse

        return AsyncTopicsResourceWithRawResponse(self._client.topics)

    @cached_property
    def issues(self) -> issues.AsyncIssuesResourceWithRawResponse:
        from .resources.issues import AsyncIssuesResourceWithRawResponse

        return AsyncIssuesResourceWithRawResponse(self._client.issues)

    @cached_property
    def style_guides(self) -> style_guides.AsyncStyleGuidesResourceWithRawResponse:
        from .resources.style_guides import AsyncStyleGuidesResourceWithRawResponse

        return AsyncStyleGuidesResourceWithRawResponse(self._client.style_guides)

    @cached_property
    def documents(self) -> documents.AsyncDocumentsResourceWithRawResponse:
        from .resources.documents import AsyncDocumentsResourceWithRawResponse

        return AsyncDocumentsResourceWithRawResponse(self._client.documents)

    @cached_property
    def tags(self) -> tags.AsyncTagsResourceWithRawResponse:
        from .resources.tags import AsyncTagsResourceWithRawResponse

        return AsyncTagsResourceWithRawResponse(self._client.tags)


class AvidoWithStreamedResponse:
    _client: Avido

    def __init__(self, client: Avido) -> None:
        self._client = client

    @cached_property
    def validate_webhook(self) -> validate_webhook.ValidateWebhookResourceWithStreamingResponse:
        from .resources.validate_webhook import ValidateWebhookResourceWithStreamingResponse

        return ValidateWebhookResourceWithStreamingResponse(self._client.validate_webhook)

    @cached_property
    def applications(self) -> applications.ApplicationsResourceWithStreamingResponse:
        from .resources.applications import ApplicationsResourceWithStreamingResponse

        return ApplicationsResourceWithStreamingResponse(self._client.applications)

    @cached_property
    def traces(self) -> traces.TracesResourceWithStreamingResponse:
        from .resources.traces import TracesResourceWithStreamingResponse

        return TracesResourceWithStreamingResponse(self._client.traces)

    @cached_property
    def ingest(self) -> ingest.IngestResourceWithStreamingResponse:
        from .resources.ingest import IngestResourceWithStreamingResponse

        return IngestResourceWithStreamingResponse(self._client.ingest)

    @cached_property
    def tasks(self) -> tasks.TasksResourceWithStreamingResponse:
        from .resources.tasks import TasksResourceWithStreamingResponse

        return TasksResourceWithStreamingResponse(self._client.tasks)

    @cached_property
    def definitions(self) -> definitions.DefinitionsResourceWithStreamingResponse:
        from .resources.definitions import DefinitionsResourceWithStreamingResponse

        return DefinitionsResourceWithStreamingResponse(self._client.definitions)

    @cached_property
    def tests(self) -> tests.TestsResourceWithStreamingResponse:
        from .resources.tests import TestsResourceWithStreamingResponse

        return TestsResourceWithStreamingResponse(self._client.tests)

    @cached_property
    def topics(self) -> topics.TopicsResourceWithStreamingResponse:
        from .resources.topics import TopicsResourceWithStreamingResponse

        return TopicsResourceWithStreamingResponse(self._client.topics)

    @cached_property
    def issues(self) -> issues.IssuesResourceWithStreamingResponse:
        from .resources.issues import IssuesResourceWithStreamingResponse

        return IssuesResourceWithStreamingResponse(self._client.issues)

    @cached_property
    def style_guides(self) -> style_guides.StyleGuidesResourceWithStreamingResponse:
        from .resources.style_guides import StyleGuidesResourceWithStreamingResponse

        return StyleGuidesResourceWithStreamingResponse(self._client.style_guides)

    @cached_property
    def documents(self) -> documents.DocumentsResourceWithStreamingResponse:
        from .resources.documents import DocumentsResourceWithStreamingResponse

        return DocumentsResourceWithStreamingResponse(self._client.documents)

    @cached_property
    def tags(self) -> tags.TagsResourceWithStreamingResponse:
        from .resources.tags import TagsResourceWithStreamingResponse

        return TagsResourceWithStreamingResponse(self._client.tags)


class AsyncAvidoWithStreamedResponse:
    _client: AsyncAvido

    def __init__(self, client: AsyncAvido) -> None:
        self._client = client

    @cached_property
    def validate_webhook(self) -> validate_webhook.AsyncValidateWebhookResourceWithStreamingResponse:
        from .resources.validate_webhook import AsyncValidateWebhookResourceWithStreamingResponse

        return AsyncValidateWebhookResourceWithStreamingResponse(self._client.validate_webhook)

    @cached_property
    def applications(self) -> applications.AsyncApplicationsResourceWithStreamingResponse:
        from .resources.applications import AsyncApplicationsResourceWithStreamingResponse

        return AsyncApplicationsResourceWithStreamingResponse(self._client.applications)

    @cached_property
    def traces(self) -> traces.AsyncTracesResourceWithStreamingResponse:
        from .resources.traces import AsyncTracesResourceWithStreamingResponse

        return AsyncTracesResourceWithStreamingResponse(self._client.traces)

    @cached_property
    def ingest(self) -> ingest.AsyncIngestResourceWithStreamingResponse:
        from .resources.ingest import AsyncIngestResourceWithStreamingResponse

        return AsyncIngestResourceWithStreamingResponse(self._client.ingest)

    @cached_property
    def tasks(self) -> tasks.AsyncTasksResourceWithStreamingResponse:
        from .resources.tasks import AsyncTasksResourceWithStreamingResponse

        return AsyncTasksResourceWithStreamingResponse(self._client.tasks)

    @cached_property
    def definitions(self) -> definitions.AsyncDefinitionsResourceWithStreamingResponse:
        from .resources.definitions import AsyncDefinitionsResourceWithStreamingResponse

        return AsyncDefinitionsResourceWithStreamingResponse(self._client.definitions)

    @cached_property
    def tests(self) -> tests.AsyncTestsResourceWithStreamingResponse:
        from .resources.tests import AsyncTestsResourceWithStreamingResponse

        return AsyncTestsResourceWithStreamingResponse(self._client.tests)

    @cached_property
    def topics(self) -> topics.AsyncTopicsResourceWithStreamingResponse:
        from .resources.topics import AsyncTopicsResourceWithStreamingResponse

        return AsyncTopicsResourceWithStreamingResponse(self._client.topics)

    @cached_property
    def issues(self) -> issues.AsyncIssuesResourceWithStreamingResponse:
        from .resources.issues import AsyncIssuesResourceWithStreamingResponse

        return AsyncIssuesResourceWithStreamingResponse(self._client.issues)

    @cached_property
    def style_guides(self) -> style_guides.AsyncStyleGuidesResourceWithStreamingResponse:
        from .resources.style_guides import AsyncStyleGuidesResourceWithStreamingResponse

        return AsyncStyleGuidesResourceWithStreamingResponse(self._client.style_guides)

    @cached_property
    def documents(self) -> documents.AsyncDocumentsResourceWithStreamingResponse:
        from .resources.documents import AsyncDocumentsResourceWithStreamingResponse

        return AsyncDocumentsResourceWithStreamingResponse(self._client.documents)

    @cached_property
    def tags(self) -> tags.AsyncTagsResourceWithStreamingResponse:
        from .resources.tags import AsyncTagsResourceWithStreamingResponse

        return AsyncTagsResourceWithStreamingResponse(self._client.tags)


Client = Avido

AsyncClient = AsyncAvido



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Union, Iterable, Optional
from datetime import datetime
from typing_extensions import Literal, Required, Annotated, TypeAlias, TypedDict

from .._types import SequenceNotStr
from .._utils import PropertyInfo
from .eval_type import EvalType
from .usage_param import UsageParam

__all__ = [
    "IngestCreateParams",
    "Event",
    "EventIngestTrace",
    "EventIngestTraceEval",
    "EventIngestTraceEvalDefinition",
    "EventIngestTraceEvalDefinitionApplication",
    "EventIngestTraceEvalDefinitionGlobalConfig",
    "EventIngestTraceEvalDefinitionGlobalConfigCriterion",
    "EventIngestTraceEvalDefinitionGlobalConfigGroundTruth",
    "EventIngestTraceEvalDefinitionGlobalConfigOutputMatchStringConfig",
    "EventIngestTraceEvalDefinitionGlobalConfigOutputMatchStringConfigExtract",
    "EventIngestTraceEvalDefinitionGlobalConfigOutputMatchListConfig",
    "EventIngestTraceEvalDefinitionGlobalConfigOutputMatchListConfigExtract",
    "EventIngestTraceEvalResults",
    "EventIngestTraceEvalResultsUnionMember0",
    "EventIngestTraceEvalResultsUnionMember1",
    "EventIngestTraceEvalResultsUnionMember2",
    "EventIngestTraceEvalResultsUnionMember2AnswerRelevancy",
    "EventIngestTraceEvalResultsUnionMember2AnswerRelevancyMetadata",
    "EventIngestTraceEvalResultsUnionMember2AnswerRelevancyMetadataQuestion",
    "EventIngestTraceEvalResultsUnionMember2AnswerRelevancyMetadataSimilarity",
    "EventIngestTraceEvalResultsUnionMember2ContextPrecision",
    "EventIngestTraceEvalResultsUnionMember2ContextPrecisionMetadata",
    "EventIngestTraceEvalResultsUnionMember2ContextRelevancy",
    "EventIngestTraceEvalResultsUnionMember2ContextRelevancyMetadata",
    "EventIngestTraceEvalResultsUnionMember2ContextRelevancyMetadataRelevantSentence",
    "EventIngestTraceEvalResultsUnionMember2Faithfulness",
    "EventIngestTraceEvalResultsUnionMember2FaithfulnessMetadata",
    "EventIngestTraceEvalResultsUnionMember2FaithfulnessMetadataFaithfulness",
    "EventIngestTraceEvalResultsUnionMember3",
    "EventIngestTraceEvalResultsUnionMember3Metadata",
    "EventIngestTraceEvalResultsUnionMember4",
    "EventIngestTraceEvalResultsUnionMember4Classification",
    "EventIngestTraceEvalResultsUnionMember5",
    "EventIngestTraceEvalResultsUnionMember5Missing",
    "EventIngestTraceStep",
    "EventIngestTraceStepLlmStartStep",
    "EventIngestTraceStepLlmEndStep",
    "EventIngestTraceStepToolStep",
    "EventIngestTraceStepRetrieverStep",
    "EventIngestTraceStepLogStep",
    "EventIngestTraceStepGroupStep",
    "EventIngestTraceStepResponseStep",
    "EventIngestTraceStepRequestStep",
    "EventIngestLlmStartStep",
    "EventIngestLlmEndStep",
    "EventIngestToolStep",
    "EventIngestRetrieverStep",
    "EventIngestLogStep",
    "EventIngestGroupStep",
    "EventIngestResponseStep",
    "EventIngestRequestStep",
]


class IngestCreateParams(TypedDict, total=False):
    events: Required[Iterable[Event]]
    """Array of events to be ingested, which can be traces or steps."""


class EventIngestTraceEvalDefinitionApplication(TypedDict, total=False):
    """Application configuration and metadata"""

    id: Required[str]
    """Unique identifier of the application"""

    context: Required[str]
    """Context/instructions for the application"""

    created_at: Required[Annotated[Union[str, datetime], PropertyInfo(alias="createdAt", format="iso8601")]]
    """When the application was created"""

    description: Required[str]
    """Description of the application"""

    language: Required[str]
    """Language of the application"""

    modified_at: Required[Annotated[Union[str, datetime], PropertyInfo(alias="modifiedAt", format="iso8601")]]
    """When the application was last modified"""

    org_id: Required[Annotated[str, PropertyInfo(alias="orgId")]]
    """Organization ID that owns this application"""

    slug: Required[str]
    """URL-friendly slug for the application"""

    title: Required[str]
    """Title of the application"""

    type: Required[Literal["CHATBOT", "AGENT"]]
    """Type of the application. Valid values are CHATBOT or AGENT."""

    environment: Literal["DEV", "PROD"]
    """Environment of the application. Defaults to DEV."""


class EventIngestTraceEvalDefinitionGlobalConfigCriterion(TypedDict, total=False):
    criterion: Required[str]
    """The criterion describes what our evaluation LLM must look for in the response.

    Remember that the answer to the criterion must be as a pass/fail.
    """


class EventIngestTraceEvalDefinitionGlobalConfigGroundTruth(TypedDict, total=False):
    ground_truth: Required[Annotated[str, PropertyInfo(alias="groundTruth")]]
    """
    The ground truth is the most correct answer to the task that we measure the
    response against
    """


class EventIngestTraceEvalDefinitionGlobalConfigOutputMatchStringConfigExtract(TypedDict, total=False):
    flags: Required[str]

    group: Required[Union[int, Iterable[int]]]

    pattern: Required[str]


class EventIngestTraceEvalDefinitionGlobalConfigOutputMatchStringConfig(TypedDict, total=False):
    type: Required[Literal["string"]]

    extract: EventIngestTraceEvalDefinitionGlobalConfigOutputMatchStringConfigExtract


class EventIngestTraceEvalDefinitionGlobalConfigOutputMatchListConfigExtract(TypedDict, total=False):
    flags: Required[str]

    group: Required[Union[int, Iterable[int]]]

    pattern: Required[str]


class EventIngestTraceEvalDefinitionGlobalConfigOutputMatchListConfig(TypedDict, total=False):
    match_mode: Required[Annotated[Literal["exact_unordered", "contains"], PropertyInfo(alias="matchMode")]]

    type: Required[Literal["list"]]

    extract: EventIngestTraceEvalDefinitionGlobalConfigOutputMatchListConfigExtract

    pass_threshold: Annotated[float, PropertyInfo(alias="passThreshold")]

    score_metric: Annotated[Literal["f1", "precision", "recall"], PropertyInfo(alias="scoreMetric")]


EventIngestTraceEvalDefinitionGlobalConfig: TypeAlias = Union[
    EventIngestTraceEvalDefinitionGlobalConfigCriterion,
    EventIngestTraceEvalDefinitionGlobalConfigGroundTruth,
    EventIngestTraceEvalDefinitionGlobalConfigOutputMatchStringConfig,
    EventIngestTraceEvalDefinitionGlobalConfigOutputMatchListConfig,
]


class EventIngestTraceEvalDefinition(TypedDict, total=False):
    id: Required[str]

    created_at: Required[Annotated[Union[str, datetime], PropertyInfo(alias="createdAt", format="iso8601")]]
    """When the eval definition was created"""

    modified_at: Required[Annotated[Union[str, datetime], PropertyInfo(alias="modifiedAt", format="iso8601")]]
    """When the eval definition was last modified"""

    name: Required[str]

    type: Required[EvalType]

    application: Optional[EventIngestTraceEvalDefinitionApplication]
    """Application configuration and metadata"""

    global_config: Annotated[EventIngestTraceEvalDefinitionGlobalConfig, PropertyInfo(alias="globalConfig")]

    style_guide_id: Annotated[Optional[str], PropertyInfo(alias="styleGuideId")]


class EventIngestTraceEvalResultsUnionMember0(TypedDict, total=False):
    analysis: Required[str]

    clarity: Required[float]

    coherence: Required[float]

    engagingness: Required[float]

    naturalness: Required[float]

    relevance: Required[float]


class EventIngestTraceEvalResultsUnionMember1(TypedDict, total=False):
    analysis: Required[str]
    """
    A brief explanation for your rating, referring to specific aspects of the
    response and the query. Make sure that the explanation is formatted as markdown,
    and that it is easy to read and understand.
    """

    score: Required[float]
    """The score of the response based on the style guide from 1-5"""


class EventIngestTraceEvalResultsUnionMember2AnswerRelevancyMetadataQuestion(TypedDict, total=False):
    question: Required[str]


class EventIngestTraceEvalResultsUnionMember2AnswerRelevancyMetadataSimilarity(TypedDict, total=False):
    question: Required[str]

    score: Required[float]


class EventIngestTraceEvalResultsUnionMember2AnswerRelevancyMetadata(TypedDict, total=False):
    questions: Required[Iterable[EventIngestTraceEvalResultsUnionMember2AnswerRelevancyMetadataQuestion]]

    similarity: Required[Iterable[EventIngestTraceEvalResultsUnionMember2AnswerRelevancyMetadataSimilarity]]


class EventIngestTraceEvalResultsUnionMember2AnswerRelevancy(TypedDict, total=False):
    name: Required[str]

    score: Required[float]

    error: str

    metadata: EventIngestTraceEvalResultsUnionMember2AnswerRelevancyMetadata


class EventIngestTraceEvalResultsUnionMember2ContextPrecisionMetadata(TypedDict, total=False):
    reason: Required[str]

    verdict: Required[int]


class EventIngestTraceEvalResultsUnionMember2ContextPrecision(TypedDict, total=False):
    name: Required[str]

    score: Required[float]

    error: str

    metadata: EventIngestTraceEvalResultsUnionMember2ContextPrecisionMetadata


class EventIngestTraceEvalResultsUnionMember2ContextRelevancyMetadataRelevantSentence(TypedDict, total=False):
    reasons: Required[SequenceNotStr[str]]

    sentence: Required[str]


class EventIngestTraceEvalResultsUnionMember2ContextRelevancyMetadata(TypedDict, total=False):
    relevant_sentences: Required[
        Annotated[
            Iterable[EventIngestTraceEvalResultsUnionMember2ContextRelevancyMetadataRelevantSentence],
            PropertyInfo(alias="relevantSentences"),
        ]
    ]


class EventIngestTraceEvalResultsUnionMember2ContextRelevancy(TypedDict, total=False):
    name: Required[str]

    score: Required[float]

    error: str

    metadata: EventIngestTraceEvalResultsUnionMember2ContextRelevancyMetadata


class EventIngestTraceEvalResultsUnionMember2FaithfulnessMetadataFaithfulness(TypedDict, total=False):
    reason: Required[str]

    statement: Required[str]

    verdict: Required[int]

    classification: Literal["UNSUPPORTED_CLAIM", "CONTRADICTION", "PARTIAL_HALLUCINATION", "SCOPE_DRIFT"]
    """Classification of the hallucination type (only present when verdict is 0)"""


class EventIngestTraceEvalResultsUnionMember2FaithfulnessMetadata(TypedDict, total=False):
    faithfulness: Required[Iterable[EventIngestTraceEvalResultsUnionMember2FaithfulnessMetadataFaithfulness]]

    statements: Required[SequenceNotStr[str]]


class EventIngestTraceEvalResultsUnionMember2Faithfulness(TypedDict, total=False):
    name: Required[str]

    score: Required[float]

    error: str

    metadata: EventIngestTraceEvalResultsUnionMember2FaithfulnessMetadata


class EventIngestTraceEvalResultsUnionMember2(TypedDict, total=False):
    answer_relevancy: Required[
        Annotated[EventIngestTraceEvalResultsUnionMember2AnswerRelevancy, PropertyInfo(alias="AnswerRelevancy")]
    ]

    context_precision: Required[
        Annotated[EventIngestTraceEvalResultsUnionMember2ContextPrecision, PropertyInfo(alias="ContextPrecision")]
    ]

    context_relevancy: Required[
        Annotated[EventIngestTraceEvalResultsUnionMember2ContextRelevancy, PropertyInfo(alias="ContextRelevancy")]
    ]

    faithfulness: Required[
        Annotated[EventIngestTraceEvalResultsUnionMember2Faithfulness, PropertyInfo(alias="Faithfulness")]
    ]


class EventIngestTraceEvalResultsUnionMember3Metadata(TypedDict, total=False):
    rationale: Required[str]


class EventIngestTraceEvalResultsUnionMember3(TypedDict, total=False):
    name: Required[str]

    score: Required[int]

    error: str

    metadata: EventIngestTraceEvalResultsUnionMember3Metadata


class EventIngestTraceEvalResultsUnionMember4Classification(TypedDict, total=False):
    fn: Required[Annotated[SequenceNotStr[str], PropertyInfo(alias="FN")]]
    """False negatives: Statements found in the ground truth but omitted in the answer"""

    fp: Required[Annotated[SequenceNotStr[str], PropertyInfo(alias="FP")]]
    """
    False positives: Statements present in the answer but not found in the ground
    truth
    """

    tp: Required[Annotated[SequenceNotStr[str], PropertyInfo(alias="TP")]]
    """
    True positives: Statements that are present in both the answer and the ground
    truth
    """


class EventIngestTraceEvalResultsUnionMember4(TypedDict, total=False):
    classification: Required[EventIngestTraceEvalResultsUnionMember4Classification]

    score: Required[float]
    """The F1 score of the response based on the fact checker (0-1)"""

    error: str

    metadata: object


class EventIngestTraceEvalResultsUnionMember5Missing(TypedDict, total=False):
    have: Required[float]

    need: Required[float]

    value: Required[str]


class EventIngestTraceEvalResultsUnionMember5(TypedDict, total=False):
    reason: Required[str]

    score: Required[float]

    expected: str

    expected_list: Annotated[SequenceNotStr[str], PropertyInfo(alias="expectedList")]

    f1: float

    got: str

    got_list: Annotated[SequenceNotStr[str], PropertyInfo(alias="gotList")]

    match_mode: Annotated[Literal["exact_unordered", "contains"], PropertyInfo(alias="matchMode")]

    missing: Iterable[EventIngestTraceEvalResultsUnionMember5Missing]

    precision: float

    recall: float

    score_metric: Annotated[Literal["f1", "precision", "recall"], PropertyInfo(alias="scoreMetric")]

    tp: float


EventIngestTraceEvalResults: TypeAlias = Union[
    EventIngestTraceEvalResultsUnionMember0,
    EventIngestTraceEvalResultsUnionMember1,
    EventIngestTraceEvalResultsUnionMember2,
    EventIngestTraceEvalResultsUnionMember3,
    EventIngestTraceEvalResultsUnionMember4,
    EventIngestTraceEvalResultsUnionMember5,
]


class EventIngestTraceEval(TypedDict, total=False):
    """Complete evaluation information"""

    id: Required[str]
    """Unique identifier of the evaluation"""

    created_at: Required[Annotated[Union[str, datetime], PropertyInfo(alias="createdAt", format="iso8601")]]
    """When the evaluation was created"""

    definition: Required[EventIngestTraceEvalDefinition]

    modified_at: Required[Annotated[Union[str, datetime], PropertyInfo(alias="modifiedAt", format="iso8601")]]
    """When the evaluation was last modified"""

    org_id: Required[Annotated[str, PropertyInfo(alias="orgId")]]
    """Organization ID that owns this evaluation"""

    status: Required[Literal["PENDING", "IN_PROGRESS", "COMPLETED", "FAILED"]]
    """Status of the evaluation/test"""

    passed: bool
    """Whether the evaluation passed"""

    results: EventIngestTraceEvalResults
    """Results of the evaluation (structure depends on eval type)."""

    score: float
    """Overall score of the evaluation"""


class EventIngestTraceStepLlmStartStep(TypedDict, total=False):
    """Start of an LLM trace."""

    id: Required[str]
    """UUID for the step."""

    event: Required[Literal["start"]]

    input: Required[str]
    """JSON input for this LLM trace event (e.g., the prompt)."""

    model_id: Required[Annotated[str, PropertyInfo(alias="modelId")]]
    """Model ID or name used for the LLM call."""

    timestamp: Required[Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]]
    """When the step was created"""

    trace_id: Required[Annotated[str, PropertyInfo(alias="traceId")]]
    """UUID referencing the parent trace's ID."""

    type: Required[Literal["llm"]]

    cost_amount: Annotated[float, PropertyInfo(alias="costAmount")]
    """Cost of the step in the configured currency (primarily for LLM steps)"""

    duration_ms: Annotated[int, PropertyInfo(alias="durationMs")]
    """Duration of the step in milliseconds"""

    end_time: Annotated[Union[str, datetime], PropertyInfo(alias="endTime", format="iso8601")]
    """When the step ended (for duration tracking)"""

    error: str
    """Error message or stack trace if the step failed."""

    group: str
    """Optionally, the key for a group step to group the step with."""

    metadata: Union[Dict[str, object], Iterable[object], str]
    """Extra metadata about this trace event.

    String values are parsed as JSON if possible, otherwise wrapped in { raw: val }.
    """

    name: str
    """The name of the step."""

    params: Union[Dict[str, object], Iterable[object], str]
    """Arbitrary params for the step."""

    parent_id: Annotated[str, PropertyInfo(alias="parentId")]
    """Parent step ID for hierarchical span relationships.

    Links this step to its parent in the trace tree.
    """

    status: Literal["success", "error", "timeout", "pending"]
    """Status of the step indicating success or failure. Defaults to 'success'."""

    status_code: Annotated[str, PropertyInfo(alias="statusCode")]
    """
    Status code associated with the step (e.g., HTTP status code for API calls, OTEL
    status code for traced spans).
    """


class EventIngestTraceStepLlmEndStep(TypedDict, total=False):
    """End of an LLM trace."""

    id: Required[str]
    """UUID for the step."""

    event: Required[Literal["end"]]

    model_id: Required[Annotated[str, PropertyInfo(alias="modelId")]]
    """Model ID or name used for the LLM call."""

    timestamp: Required[Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]]
    """When the step was created"""

    trace_id: Required[Annotated[str, PropertyInfo(alias="traceId")]]
    """UUID referencing the parent trace's ID."""

    type: Required[Literal["llm"]]

    cost_amount: Annotated[float, PropertyInfo(alias="costAmount")]
    """Cost of the step in the configured currency (primarily for LLM steps)"""

    duration_ms: Annotated[int, PropertyInfo(alias="durationMs")]
    """Duration of the step in milliseconds"""

    end_time: Annotated[Union[str, datetime], PropertyInfo(alias="endTime", format="iso8601")]
    """When the step ended (for duration tracking)"""

    error: str
    """Error message or stack trace if the step failed."""

    finish_reason: Annotated[str, PropertyInfo(alias="finishReason")]
    """
    The reason the LLM stopped generating, extracted from
    gen_ai.response.finish_reasons.
    """

    group: str
    """Optionally, the key for a group step to group the step with."""

    metadata: Union[Dict[str, object], Iterable[object], str]
    """Extra metadata about this trace event.

    String values are parsed as JSON if possible, otherwise wrapped in { raw: val }.
    """

    name: str
    """The name of the step."""

    output: str
    """JSON describing the output.

    String inputs are parsed or wrapped in { message: val }.
    """

    params: Union[Dict[str, object], Iterable[object], str]
    """Arbitrary params for the step."""

    parent_id: Annotated[str, PropertyInfo(alias="parentId")]
    """Parent step ID for hierarchical span relationships.

    Links this step to its parent in the trace tree.
    """

    status: Literal["success", "error", "timeout", "pending"]
    """Status of the step indicating success or failure. Defaults to 'success'."""

    status_code: Annotated[str, PropertyInfo(alias="statusCode")]
    """
    Status code associated with the step (e.g., HTTP status code for API calls, OTEL
    status code for traced spans).
    """

    usage: UsageParam
    """Number of input and output tokens used by the LLM."""


class EventIngestTraceStepToolStep(TypedDict, total=False):
    """Track all tool calls using the Tool Step event"""

    id: Required[str]
    """UUID for the step."""

    timestamp: Required[Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]]
    """When the step was created"""

    trace_id: Required[Annotated[str, PropertyInfo(alias="traceId")]]
    """UUID referencing the parent trace's ID."""

    type: Required[Literal["tool"]]

    cost_amount: Annotated[float, PropertyInfo(alias="costAmount")]
    """Cost of the step in the configured currency (primarily for LLM steps)"""

    duration_ms: Annotated[int, PropertyInfo(alias="durationMs")]
    """Duration of the step in milliseconds"""

    end_time: Annotated[Union[str, datetime], PropertyInfo(alias="endTime", format="iso8601")]
    """When the step ended (for duration tracking)"""

    error: str
    """Error message or stack trace if the step failed."""

    group: str
    """Optionally, the key for a group step to group the step with."""

    metadata: Union[Dict[str, object], Iterable[object], str]
    """Extra metadata about this trace event.

    String values are parsed as JSON if possible, otherwise wrapped in { raw: val }.
    """

    name: str
    """The name of the step."""

    params: Union[Dict[str, object], Iterable[object], str]
    """Arbitrary params for the step."""

    parent_id: Annotated[str, PropertyInfo(alias="parentId")]
    """Parent step ID for hierarchical span relationships.

    Links this step to its parent in the trace tree.
    """

    status: Literal["success", "error", "timeout", "pending"]
    """Status of the step indicating success or failure. Defaults to 'success'."""

    status_code: Annotated[str, PropertyInfo(alias="statusCode")]
    """
    Status code associated with the step (e.g., HTTP status code for API calls, OTEL
    status code for traced spans).
    """

    tool_call_id: Annotated[str, PropertyInfo(alias="toolCallId")]
    """
    Correlation ID from the LLM's tool_calls response (e.g., OpenAI's call_abc123 or
    Claude's toolu_01A09q). Links tool execution to the originating LLM request in
    parallel tool call scenarios.
    """

    tool_input: Annotated[str, PropertyInfo(alias="toolInput")]
    """JSON input for the tool call."""

    tool_output: Annotated[str, PropertyInfo(alias="toolOutput")]
    """JSON output from the tool call."""


class EventIngestTraceStepRetrieverStep(TypedDict, total=False):
    """Track all retriever (RAG) calls using the Retriever Step event."""

    id: Required[str]
    """UUID for the step."""

    query: Required[str]
    """Query used for RAG."""

    result: Required[str]
    """Retrieved text"""

    timestamp: Required[Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]]
    """When the step was created"""

    trace_id: Required[Annotated[str, PropertyInfo(alias="traceId")]]
    """UUID referencing the parent trace's ID."""

    type: Required[Literal["retriever"]]

    cost_amount: Annotated[float, PropertyInfo(alias="costAmount")]
    """Cost of the step in the configured currency (primarily for LLM steps)"""

    duration_ms: Annotated[int, PropertyInfo(alias="durationMs")]
    """Duration of the step in milliseconds"""

    end_time: Annotated[Union[str, datetime], PropertyInfo(alias="endTime", format="iso8601")]
    """When the step ended (for duration tracking)"""

    error: str
    """Error message or stack trace if the step failed."""

    group: str
    """Optionally, the key for a group step to group the step with."""

    metadata: Union[Dict[str, object], Iterable[object], str]
    """Extra metadata about this trace event.

    String values are parsed as JSON if possible, otherwise wrapped in { raw: val }.
    """

    name: str
    """The name of the step."""

    params: Union[Dict[str, object], Iterable[object], str]
    """Arbitrary params for the step."""

    parent_id: Annotated[str, PropertyInfo(alias="parentId")]
    """Parent step ID for hierarchical span relationships.

    Links this step to its parent in the trace tree.
    """

    status: Literal["success", "error", "timeout", "pending"]
    """Status of the step indicating success or failure. Defaults to 'success'."""

    status_code: Annotated[str, PropertyInfo(alias="statusCode")]
    """
    Status code associated with the step (e.g., HTTP status code for API calls, OTEL
    status code for traced spans).
    """


class EventIngestTraceStepLogStep(TypedDict, total=False):
    """Track all logs using the Log Step event."""

    id: Required[str]
    """UUID for the step."""

    content: Required[str]
    """The actual log message for this trace."""

    timestamp: Required[Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]]
    """When the step was created"""

    trace_id: Required[Annotated[str, PropertyInfo(alias="traceId")]]
    """UUID referencing the parent trace's ID."""

    type: Required[Literal["log"]]

    cost_amount: Annotated[float, PropertyInfo(alias="costAmount")]
    """Cost of the step in the configured currency (primarily for LLM steps)"""

    duration_ms: Annotated[int, PropertyInfo(alias="durationMs")]
    """Duration of the step in milliseconds"""

    end_time: Annotated[Union[str, datetime], PropertyInfo(alias="endTime", format="iso8601")]
    """When the step ended (for duration tracking)"""

    error: str
    """Error message or stack trace if the step failed."""

    group: str
    """Optionally, the key for a group step to group the step with."""

    metadata: Union[Dict[str, object], Iterable[object], str]
    """Extra metadata about this trace event.

    String values are parsed as JSON if possible, otherwise wrapped in { raw: val }.
    """

    name: str
    """The name of the step."""

    params: Union[Dict[str, object], Iterable[object], str]
    """Arbitrary params for the step."""

    parent_id: Annotated[str, PropertyInfo(alias="parentId")]
    """Parent step ID for hierarchical span relationships.

    Links this step to its parent in the trace tree.
    """

    status: Literal["success", "error", "timeout", "pending"]
    """Status of the step indicating success or failure. Defaults to 'success'."""

    status_code: Annotated[str, PropertyInfo(alias="statusCode")]
    """
    Status code associated with the step (e.g., HTTP status code for API calls, OTEL
    status code for traced spans).
    """


class EventIngestTraceStepGroupStep(TypedDict, total=False):
    """
    Use this to group multiple steps together, for example a log, llm start, and llm end.
    """

    id: Required[str]
    """UUID for the step."""

    key: Required[str]
    """
    A unique identifier for the grouping, which must be appended to the
    corresponding steps
    """

    timestamp: Required[Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]]
    """When the step was created"""

    trace_id: Required[Annotated[str, PropertyInfo(alias="traceId")]]
    """UUID referencing the parent trace's ID."""

    type: Required[Literal["group"]]

    cost_amount: Annotated[float, PropertyInfo(alias="costAmount")]
    """Cost of the step in the configured currency (primarily for LLM steps)"""

    duration_ms: Annotated[int, PropertyInfo(alias="durationMs")]
    """Duration of the step in milliseconds"""

    end_time: Annotated[Union[str, datetime], PropertyInfo(alias="endTime", format="iso8601")]
    """When the step ended (for duration tracking)"""

    error: str
    """Error message or stack trace if the step failed."""

    group: str
    """Optionally, the key for a group step to group the step with."""

    metadata: Union[Dict[str, object], Iterable[object], str]
    """Extra metadata about this trace event.

    String values are parsed as JSON if possible, otherwise wrapped in { raw: val }.
    """

    name: str
    """The name of the step."""

    params: Union[Dict[str, object], Iterable[object], str]
    """Arbitrary params for the step."""

    parent_id: Annotated[str, PropertyInfo(alias="parentId")]
    """Parent step ID for hierarchical span relationships.

    Links this step to its parent in the trace tree.
    """

    status: Literal["success", "error", "timeout", "pending"]
    """Status of the step indicating success or failure. Defaults to 'success'."""

    status_code: Annotated[str, PropertyInfo(alias="statusCode")]
    """
    Status code associated with the step (e.g., HTTP status code for API calls, OTEL
    status code for traced spans).
    """


class EventIngestTraceStepResponseStep(TypedDict, total=False):
    """Track AI responses to users using the Response Step event."""

    id: Required[str]
    """UUID for the step."""

    content: Required[str]
    """The response content from the AI to the user."""

    timestamp: Required[Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]]
    """When the step was created"""

    trace_id: Required[Annotated[str, PropertyInfo(alias="traceId")]]
    """UUID referencing the parent trace's ID."""

    type: Required[Literal["response"]]

    cost_amount: Annotated[float, PropertyInfo(alias="costAmount")]
    """Cost of the step in the configured currency (primarily for LLM steps)"""

    duration_ms: Annotated[int, PropertyInfo(alias="durationMs")]
    """Duration of the step in milliseconds"""

    end_time: Annotated[Union[str, datetime], PropertyInfo(alias="endTime", format="iso8601")]
    """When the step ended (for duration tracking)"""

    error: str
    """Error message or stack trace if the step failed."""

    group: str
    """Optionally, the key for a group step to group the step with."""

    metadata: Union[Dict[str, object], Iterable[object], str]
    """Extra metadata about this trace event.

    String values are parsed as JSON if possible, otherwise wrapped in { raw: val }.
    """

    name: str
    """The name of the step."""

    params: Union[Dict[str, object], Iterable[object], str]
    """Arbitrary params for the step."""

    parent_id: Annotated[str, PropertyInfo(alias="parentId")]
    """Parent step ID for hierarchical span relationships.

    Links this step to its parent in the trace tree.
    """

    status: Literal["success", "error", "timeout", "pending"]
    """Status of the step indicating success or failure. Defaults to 'success'."""

    status_code: Annotated[str, PropertyInfo(alias="statusCode")]
    """
    Status code associated with the step (e.g., HTTP status code for API calls, OTEL
    status code for traced spans).
    """


class EventIngestTraceStepRequestStep(TypedDict, total=False):
    """Track user requests to the AI using the Request Step event."""

    id: Required[str]
    """UUID for the step."""

    content: Required[str]
    """The request content from the user to the AI."""

    timestamp: Required[Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]]
    """When the step was created"""

    trace_id: Required[Annotated[str, PropertyInfo(alias="traceId")]]
    """UUID referencing the parent trace's ID."""

    type: Required[Literal["request"]]

    cost_amount: Annotated[float, PropertyInfo(alias="costAmount")]
    """Cost of the step in the configured currency (primarily for LLM steps)"""

    duration_ms: Annotated[int, PropertyInfo(alias="durationMs")]
    """Duration of the step in milliseconds"""

    end_time: Annotated[Union[str, datetime], PropertyInfo(alias="endTime", format="iso8601")]
    """When the step ended (for duration tracking)"""

    error: str
    """Error message or stack trace if the step failed."""

    group: str
    """Optionally, the key for a group step to group the step with."""

    metadata: Union[Dict[str, object], Iterable[object], str]
    """Extra metadata about this trace event.

    String values are parsed as JSON if possible, otherwise wrapped in { raw: val }.
    """

    name: str
    """The name of the step."""

    params: Union[Dict[str, object], Iterable[object], str]
    """Arbitrary params for the step."""

    parent_id: Annotated[str, PropertyInfo(alias="parentId")]
    """Parent step ID for hierarchical span relationships.

    Links this step to its parent in the trace tree.
    """

    status: Literal["success", "error", "timeout", "pending"]
    """Status of the step indicating success or failure. Defaults to 'success'."""

    status_code: Annotated[str, PropertyInfo(alias="statusCode")]
    """
    Status code associated with the step (e.g., HTTP status code for API calls, OTEL
    status code for traced spans).
    """


EventIngestTraceStep: TypeAlias = Union[
    EventIngestTraceStepLlmStartStep,
    EventIngestTraceStepLlmEndStep,
    EventIngestTraceStepToolStep,
    EventIngestTraceStepRetrieverStep,
    EventIngestTraceStepLogStep,
    EventIngestTraceStepGroupStep,
    EventIngestTraceStepResponseStep,
    EventIngestTraceStepRequestStep,
]


class EventIngestTrace(TypedDict, total=False):
    timestamp: Required[Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]]
    """When the trace was created"""

    type: Required[Literal["trace"]]

    evals: Iterable[EventIngestTraceEval]
    """Evaluation results for this trace, joined through the associated test."""

    has_error: Annotated[bool, PropertyInfo(alias="hasError")]
    """Whether any step in this trace has an error status."""

    metadata: Union[str, Dict[str, object], Iterable[object]]
    """Arbitrary metadata for this trace (e.g., userId, source, etc.)."""

    reference_id: Annotated[str, PropertyInfo(alias="referenceId")]
    """
    An optional reference ID to link the trace to an existing conversation or
    interaction in your own database.
    """

    step_count: Annotated[int, PropertyInfo(alias="stepCount")]
    """Total number of steps in this trace."""

    steps: Iterable[EventIngestTraceStep]
    """The steps associated with the trace."""

    test_id: Annotated[str, PropertyInfo(alias="testId")]
    """The associated Test if this was triggered by an Avido eval"""

    total_completion_tokens: Annotated[int, PropertyInfo(alias="totalCompletionTokens")]
    """Total number of completion tokens used across all LLM steps in this trace."""

    total_cost: Annotated[float, PropertyInfo(alias="totalCost")]
    """Total cost of all steps in this trace (sum of step costAmount values)."""

    total_duration_ms: Annotated[int, PropertyInfo(alias="totalDurationMs")]
    """Total duration of the trace in milliseconds (sum of all step durations)."""

    total_prompt_tokens: Annotated[int, PropertyInfo(alias="totalPromptTokens")]
    """Total number of prompt tokens used across all LLM steps in this trace."""

    trace_id: Annotated[str, PropertyInfo(alias="traceId")]
    """Optional trace ID (UUID) for OTEL ingestion.

    If provided and a trace with this ID already exists, the existing trace will be
    reused.
    """


class EventIngestLlmStartStep(TypedDict, total=False):
    event: Required[Literal["start"]]

    input: Required[Union[str, Dict[str, object], Iterable[object]]]
    """The input for the LLM step."""

    model_id: Required[Annotated[str, PropertyInfo(alias="modelId")]]
    """Model ID or name used for the LLM call."""

    timestamp: Required[Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]]
    """When the step was created"""

    type: Required[Literal["llm"]]

    cost_amount: Annotated[float, PropertyInfo(alias="costAmount")]
    """Cost of the step in the configured currency (primarily for LLM steps)"""

    duration_ms: Annotated[int, PropertyInfo(alias="durationMs")]
    """Duration of the step in milliseconds"""

    end_time: Annotated[Union[str, datetime], PropertyInfo(alias="endTime", format="iso8601")]
    """When the step ended (for duration tracking)"""

    error: str
    """Error message or stack trace if the step failed."""

    group: str
    """Optionally, the key for a group step to group the step with."""

    metadata: Union[Dict[str, object], Iterable[object], str]
    """Extra metadata about this trace event.

    String values are parsed as JSON if possible, otherwise wrapped in { raw: val }.
    """

    name: str
    """The name of the step."""

    params: Union[Dict[str, object], Iterable[object], str]
    """Arbitrary params for the step."""

    parent_id: Annotated[str, PropertyInfo(alias="parentId")]
    """Parent step ID for hierarchical span relationships.

    Links this step to its parent in the trace tree.
    """

    status: Literal["success", "error", "timeout", "pending"]
    """Status of the step indicating success or failure. Defaults to 'success'."""

    status_code: Annotated[str, PropertyInfo(alias="statusCode")]
    """
    Status code associated with the step (e.g., HTTP status code for API calls, OTEL
    status code for traced spans).
    """

    trace_id: Annotated[str, PropertyInfo(alias="traceId")]
    """The trace ID (UUID)."""


class EventIngestLlmEndStep(TypedDict, total=False):
    event: Required[Literal["end"]]

    model_id: Required[Annotated[str, PropertyInfo(alias="modelId")]]
    """Model ID or name used for the LLM call."""

    timestamp: Required[Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]]
    """When the step was created"""

    type: Required[Literal["llm"]]

    cost_amount: Annotated[float, PropertyInfo(alias="costAmount")]
    """Cost of the step in the configured currency (primarily for LLM steps)"""

    duration_ms: Annotated[int, PropertyInfo(alias="durationMs")]
    """Duration of the step in milliseconds"""

    end_time: Annotated[Union[str, datetime], PropertyInfo(alias="endTime", format="iso8601")]
    """When the step ended (for duration tracking)"""

    error: str
    """Error message or stack trace if the step failed."""

    finish_reason: Annotated[str, PropertyInfo(alias="finishReason")]
    """
    The reason the LLM stopped generating, extracted from
    gen_ai.response.finish_reasons.
    """

    group: str
    """Optionally, the key for a group step to group the step with."""

    metadata: Union[Dict[str, object], Iterable[object], str]
    """Extra metadata about this trace event.

    String values are parsed as JSON if possible, otherwise wrapped in { raw: val }.
    """

    name: str
    """The name of the step."""

    output: Union[str, Dict[str, object], Iterable[object]]
    """The output for the LLM step."""

    params: Union[Dict[str, object], Iterable[object], str]
    """Arbitrary params for the step."""

    parent_id: Annotated[str, PropertyInfo(alias="parentId")]
    """Parent step ID for hierarchical span relationships.

    Links this step to its parent in the trace tree.
    """

    status: Literal["success", "error", "timeout", "pending"]
    """Status of the step indicating success or failure. Defaults to 'success'."""

    status_code: Annotated[str, PropertyInfo(alias="statusCode")]
    """
    Status code associated with the step (e.g., HTTP status code for API calls, OTEL
    status code for traced spans).
    """

    trace_id: Annotated[str, PropertyInfo(alias="traceId")]
    """The trace ID (UUID)."""

    usage: UsageParam
    """Number of input and output tokens used by the LLM."""


class EventIngestToolStep(TypedDict, total=False):
    timestamp: Required[Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]]
    """When the step was created"""

    type: Required[Literal["tool"]]

    cost_amount: Annotated[float, PropertyInfo(alias="costAmount")]
    """Cost of the step in the configured currency (primarily for LLM steps)"""

    duration_ms: Annotated[int, PropertyInfo(alias="durationMs")]
    """Duration of the step in milliseconds"""

    end_time: Annotated[Union[str, datetime], PropertyInfo(alias="endTime", format="iso8601")]
    """When the step ended (for duration tracking)"""

    error: str
    """Error message or stack trace if the step failed."""

    group: str
    """Optionally, the key for a group step to group the step with."""

    metadata: Union[Dict[str, object], Iterable[object], str]
    """Extra metadata about this trace event.

    String values are parsed as JSON if possible, otherwise wrapped in { raw: val }.
    """

    name: str
    """The name of the step."""

    params: Union[Dict[str, object], Iterable[object], str]
    """Arbitrary params for the step."""

    parent_id: Annotated[str, PropertyInfo(alias="parentId")]
    """Parent step ID for hierarchical span relationships.

    Links this step to its parent in the trace tree.
    """

    status: Literal["success", "error", "timeout", "pending"]
    """Status of the step indicating success or failure. Defaults to 'success'."""

    status_code: Annotated[str, PropertyInfo(alias="statusCode")]
    """
    Status code associated with the step (e.g., HTTP status code for API calls, OTEL
    status code for traced spans).
    """

    tool_call_id: Annotated[str, PropertyInfo(alias="toolCallId")]
    """
    Correlation ID from the LLM's tool_calls response (e.g., OpenAI's call_abc123 or
    Claude's toolu_01A09q). Links tool execution to the originating LLM request in
    parallel tool call scenarios.
    """

    tool_input: Annotated[Union[str, Dict[str, object], Iterable[object]], PropertyInfo(alias="toolInput")]
    """The input for the tool step."""

    tool_output: Annotated[Union[str, Dict[str, object], Iterable[object]], PropertyInfo(alias="toolOutput")]
    """The output for the tool step."""

    trace_id: Annotated[str, PropertyInfo(alias="traceId")]
    """The trace ID (UUID)."""


class EventIngestRetrieverStep(TypedDict, total=False):
    query: Required[Union[str, Dict[str, object], Iterable[object]]]
    """The query for the retriever step."""

    result: Required[Union[str, Dict[str, object], Iterable[object]]]
    """The result for the retriever step."""

    timestamp: Required[Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]]
    """When the step was created"""

    type: Required[Literal["retriever"]]

    cost_amount: Annotated[float, PropertyInfo(alias="costAmount")]
    """Cost of the step in the configured currency (primarily for LLM steps)"""

    duration_ms: Annotated[int, PropertyInfo(alias="durationMs")]
    """Duration of the step in milliseconds"""

    end_time: Annotated[Union[str, datetime], PropertyInfo(alias="endTime", format="iso8601")]
    """When the step ended (for duration tracking)"""

    error: str
    """Error message or stack trace if the step failed."""

    group: str
    """Optionally, the key for a group step to group the step with."""

    metadata: Union[Dict[str, object], Iterable[object], str]
    """Extra metadata about this trace event.

    String values are parsed as JSON if possible, otherwise wrapped in { raw: val }.
    """

    name: str
    """The name of the step."""

    params: Union[str, Dict[str, object], Iterable[object]]
    """Arbitrary params for the step."""

    parent_id: Annotated[str, PropertyInfo(alias="parentId")]
    """Parent step ID for hierarchical span relationships.

    Links this step to its parent in the trace tree.
    """

    status: Literal["success", "error", "timeout", "pending"]
    """Status of the step indicating success or failure. Defaults to 'success'."""

    status_code: Annotated[str, PropertyInfo(alias="statusCode")]
    """
    Status code associated with the step (e.g., HTTP status code for API calls, OTEL
    status code for traced spans).
    """

    trace_id: Annotated[str, PropertyInfo(alias="traceId")]
    """The trace ID (UUID)."""


class EventIngestLogStep(TypedDict, total=False):
    content: Required[str]
    """The actual log message for this trace."""

    timestamp: Required[Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]]
    """When the step was created"""

    type: Required[Literal["log"]]

    cost_amount: Annotated[float, PropertyInfo(alias="costAmount")]
    """Cost of the step in the configured currency (primarily for LLM steps)"""

    duration_ms: Annotated[int, PropertyInfo(alias="durationMs")]
    """Duration of the step in milliseconds"""

    end_time: Annotated[Union[str, datetime], PropertyInfo(alias="endTime", format="iso8601")]
    """When the step ended (for duration tracking)"""

    error: str
    """Error message or stack trace if the step failed."""

    group: str
    """Optionally, the key for a group step to group the step with."""

    metadata: Union[Dict[str, object], Iterable[object], str]
    """Extra metadata about this trace event.

    String values are parsed as JSON if possible, otherwise wrapped in { raw: val }.
    """

    name: str
    """The name of the step."""

    params: Union[Dict[str, object], Iterable[object], str]
    """Arbitrary params for the step."""

    parent_id: Annotated[str, PropertyInfo(alias="parentId")]
    """Parent step ID for hierarchical span relationships.

    Links this step to its parent in the trace tree.
    """

    status: Literal["success", "error", "timeout", "pending"]
    """Status of the step indicating success or failure. Defaults to 'success'."""

    status_code: Annotated[str, PropertyInfo(alias="statusCode")]
    """
    Status code associated with the step (e.g., HTTP status code for API calls, OTEL
    status code for traced spans).
    """

    trace_id: Annotated[str, PropertyInfo(alias="traceId")]
    """The trace ID (UUID)."""


class EventIngestGroupStep(TypedDict, total=False):
    key: Required[str]
    """
    A unique identifier for the grouping, which must be appended to the
    corresponding steps
    """

    timestamp: Required[Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]]
    """When the step was created"""

    type: Required[Literal["group"]]

    cost_amount: Annotated[float, PropertyInfo(alias="costAmount")]
    """Cost of the step in the configured currency (primarily for LLM steps)"""

    duration_ms: Annotated[int, PropertyInfo(alias="durationMs")]
    """Duration of the step in milliseconds"""

    end_time: Annotated[Union[str, datetime], PropertyInfo(alias="endTime", format="iso8601")]
    """When the step ended (for duration tracking)"""

    error: str
    """Error message or stack trace if the step failed."""

    group: str
    """Optionally, the key for a group step to group the step with."""

    metadata: Union[Dict[str, object], Iterable[object], str]
    """Extra metadata about this trace event.

    String values are parsed as JSON if possible, otherwise wrapped in { raw: val }.
    """

    name: str
    """The name of the step."""

    params: Union[Dict[str, object], Iterable[object], str]
    """Arbitrary params for the step."""

    parent_id: Annotated[str, PropertyInfo(alias="parentId")]
    """Parent step ID for hierarchical span relationships.

    Links this step to its parent in the trace tree.
    """

    status: Literal["success", "error", "timeout", "pending"]
    """Status of the step indicating success or failure. Defaults to 'success'."""

    status_code: Annotated[str, PropertyInfo(alias="statusCode")]
    """
    Status code associated with the step (e.g., HTTP status code for API calls, OTEL
    status code for traced spans).
    """

    trace_id: Annotated[str, PropertyInfo(alias="traceId")]
    """The trace ID (UUID)."""


class EventIngestResponseStep(TypedDict, total=False):
    content: Required[Union[str, Dict[str, object], Iterable[object]]]
    """The response content from the AI to the user."""

    timestamp: Required[Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]]
    """When the step was created"""

    type: Required[Literal["response"]]

    cost_amount: Annotated[float, PropertyInfo(alias="costAmount")]
    """Cost of the step in the configured currency (primarily for LLM steps)"""

    duration_ms: Annotated[int, PropertyInfo(alias="durationMs")]
    """Duration of the step in milliseconds"""

    end_time: Annotated[Union[str, datetime], PropertyInfo(alias="endTime", format="iso8601")]
    """When the step ended (for duration tracking)"""

    error: str
    """Error message or stack trace if the step failed."""

    group: str
    """Optionally, the key for a group step to group the step with."""

    metadata: Union[Dict[str, object], Iterable[object], str]
    """Extra metadata about this trace event.

    String values are parsed as JSON if possible, otherwise wrapped in { raw: val }.
    """

    name: str
    """The name of the step."""

    params: Union[Dict[str, object], Iterable[object], str]
    """Arbitrary params for the step."""

    parent_id: Annotated[str, PropertyInfo(alias="parentId")]
    """Parent step ID for hierarchical span relationships.

    Links this step to its parent in the trace tree.
    """

    status: Literal["success", "error", "timeout", "pending"]
    """Status of the step indicating success or failure. Defaults to 'success'."""

    status_code: Annotated[str, PropertyInfo(alias="statusCode")]
    """
    Status code associated with the step (e.g., HTTP status code for API calls, OTEL
    status code for traced spans).
    """

    trace_id: Annotated[str, PropertyInfo(alias="traceId")]
    """The trace ID (UUID)."""


class EventIngestRequestStep(TypedDict, total=False):
    content: Required[Union[str, Dict[str, object], Iterable[object]]]
    """The request content from the user to the AI."""

    timestamp: Required[Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]]
    """When the step was created"""

    type: Required[Literal["request"]]

    cost_amount: Annotated[float, PropertyInfo(alias="costAmount")]
    """Cost of the step in the configured currency (primarily for LLM steps)"""

    duration_ms: Annotated[int, PropertyInfo(alias="durationMs")]
    """Duration of the step in milliseconds"""

    end_time: Annotated[Union[str, datetime], PropertyInfo(alias="endTime", format="iso8601")]
    """When the step ended (for duration tracking)"""

    error: str
    """Error message or stack trace if the step failed."""

    group: str
    """Optionally, the key for a group step to group the step with."""

    metadata: Union[Dict[str, object], Iterable[object], str]
    """Extra metadata about this trace event.

    String values are parsed as JSON if possible, otherwise wrapped in { raw: val }.
    """

    name: str
    """The name of the step."""

    params: Union[Dict[str, object], Iterable[object], str]
    """Arbitrary params for the step."""

    parent_id: Annotated[str, PropertyInfo(alias="parentId")]
    """Parent step ID for hierarchical span relationships.

    Links this step to its parent in the trace tree.
    """

    status: Literal["success", "error", "timeout", "pending"]
    """Status of the step indicating success or failure. Defaults to 'success'."""

    status_code: Annotated[str, PropertyInfo(alias="statusCode")]
    """
    Status code associated with the step (e.g., HTTP status code for API calls, OTEL
    status code for traced spans).
    """

    trace_id: Annotated[str, PropertyInfo(alias="traceId")]
    """The trace ID (UUID)."""


Event: TypeAlias = Union[
    EventIngestTrace,
    EventIngestLlmStartStep,
    EventIngestLlmEndStep,
    EventIngestToolStep,
    EventIngestRetrieverStep,
    EventIngestLogStep,
    EventIngestGroupStep,
    EventIngestResponseStep,
    EventIngestRequestStep,
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable
from typing_extensions import Required, TypedDict

__all__ = ["StyleGuideUpdateParams", "Content"]


class StyleGuideUpdateParams(TypedDict, total=False):
    content: Required[Iterable[Content]]


class Content(TypedDict, total=False):
    """A section for a style guide (input with default)"""

    content: Required[str]
    """The content of the section in markdown"""

    heading: Required[str]
    """The heading of the section"""

    approved: bool
    """Whether or not the section has been approved"""
